'use strict';

const fs = require('fs');
const querystring = require('querystring');

const jsonRead = require('../utils/json-read');
const requestWrapper = require('./request-wrapper');
const ispConstants = require('../utils/isp-utils/isp-constants');
const ispUtil = require('../utils/isp-utils/isp-utility');
const CONSTANTS = require('../utils/constants');

/*
 * List of Alexa Skill Management API Service Calls.
 * These functions are thin wrappers for Alexa Skill Management APIs.
 */

module.exports = {
    // simulation
    callSimulateSkill: callSimulateSkill,
    callGetSimulation: callGetSimulation,
    callInvokeSkill: callInvokeSkill,
    // validation
    callValidateSkill: callValidateSkill,
    callGetValidation: callGetValidation,
    // skill
    callCreateSkill: callCreateSkill,
    callGetSkill: callGetSkill,
    handleGetSkillRedirect: handleGetSkillRedirect,
    callUpdateSkill: callUpdateSkill,
    callDeleteSkill: callDeleteSkill,
    callGetSkillStatus: callGetSkillStatus,
    callListSkills: callListSkills,
    // model
    callGetModel: callGetModel,
    callHeadModel: callHeadModel,
    callUpdateModel: callUpdateModel,
    // utterance transcripts
    callGetUtteranceData: callGetUtteranceData,
    // account linking
    callCreateAccountLinking: callCreateAccountLinking,
    callGetAccountLinking: callGetAccountLinking,
    callDeleteAccountLinking: callDeleteAccountLinking,
    // misc
    callListVendor: callListVendor,
    callSubmit: callSubmit,
    callWithdraw: callWithdraw,
    callEnableSkill: callEnableSkill,
    callDisableSkill: callDisableSkill,
    callGetSkillEnablement: callGetSkillEnablement,
    callGetSkillCredentials: callGetSkillCredentials,
    // private skill
    callAddPrivateDistributionAccount: callAddPrivateDistributionAccount,
    callDeletePrivateDistributionAccount: callDeletePrivateDistributionAccount,
    callListPrivateDistributionAccounts: callListPrivateDistributionAccounts,
    // ISP
    callCreateISP: callCreateISP,
    callUpdateISP: callUpdateISP,
    callAssociateISP: callAssociateISP,
    callDisassociateISP: callDisassociateISP,
    callGetISP: callGetISP,
    callListISPForSkill: callListISPForSkill,
    callListISPForVendor: callListISPForVendor,
    callListSkillsForISP: callListSkillsForISP,
    callDeleteISP: callDeleteISP,
    callResetISPEntitlement: callResetISPEntitlement,
    // catalog
    callListCatalogsByVendor: callListCatalogsByVendor,
    callCreateCatalog: callCreateCatalog,
    callGetCatalog: callGetCatalog,
    callListCatalogUploads: callListCatalogUploads,
    callCreateCatalogUpload: callCreateCatalogUpload,
    callGetCatalogUpload: callGetCatalogUpload,
    callCompleteCatalogUpload: callCompleteCatalogUpload,
    callListCatalogsBySkill: callListCatalogsBySkill,
    callAssociateCatalogWithSkill: callAssociateCatalogWithSkill,
    // Beta Testing
    callGetBetaTest: callGetBetaTest,
    callCreateBetaTest: callCreateBetaTest,
    callUpdateBetaTest: callUpdateBetaTest,
    callStartBetaTest: callStartBetaTest,
    callEndBetaTest: callEndBetaTest,
    callListBetaTesters: callListBetaTesters,
    callAddBetaTesters: callAddBetaTesters,
    callRemoveBetaTesters: callRemoveBetaTesters,
    callSendReminderToBetaTesters: callSendReminderToBetaTesters,
    callRequestFeedbackFromBetaTesters: callRequestFeedbackFromBetaTesters,
    // Skill Accuracy
    callProfileNlu: callProfileNlu
};

function callValidateSkill(skillId, locales, stage, profile, doDebug, callback) {
    // comma or space deliminater regex
    let splitRegex = /[\s,]+/;

    locales = locales.split(splitRegex);
    let payload = {
        'locales': locales
    };

    let general = {
        url: '/skills/' + skillId + '/stages/' + stage + '/validations',
        method: 'POST'
    };
    let headers = {};
    requestWrapper.request('validate-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

function callGetValidation(validationId, skillId, stage, profile, doDebug, callback) {
    let general = {
        url: '/skills/' + skillId + '/stages/' + stage + '/validations/' + validationId,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-validation', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callInvokeSkill(file, jsonObject, skillId, endpointRegion, stage, profile, doDebug, callback) {
    const requestPayload = file ? jsonRead.readFile(file) : jsonObject;
    const invokeRequestPayload = {
        "endpointRegion": endpointRegion,
        "skillRequest": {
            "body": requestPayload
        }
    };

    const general = {
        url: '/skills/' + skillId + '/stages/' + stage + '/invocations',
        method: 'POST'
    };
    const headers = {};
    requestWrapper.request('invoke-skill', CONSTANTS.SMAPI.VERSION.V2, general, headers, invokeRequestPayload, profile, doDebug, callback);
}

function callSimulateSkill(callerCommand, file, text, skillId, locale, newSession, stage, profile, doDebug, callback) {
    const payload = {
        "input": {
            "content": file ? fs.readFileSync(file, { encoding: 'utf-8' }) : text
        },
        "device": {
            "locale": locale
        }
    };

    if (newSession) {
        payload.session = {
            mode: 'FORCE_NEW_SESSION'
        };
    }
    
    const general = {
        url: '/skills/' + skillId + '/stages/' + stage + '/simulations',
        method: 'POST'
    };
    let headers = {};
    const meta = {
        apiName: 'simulate-skill',
        callerCommand: callerCommand
    }
    requestWrapper.request(meta, CONSTANTS.SMAPI.VERSION.V2, general, headers, payload, profile, doDebug, callback);
}

function callGetSimulation(callerCommand, simulationId, skillId, stage, profile, doDebug, callback) {
    const general = {
        url: '/skills/' + skillId + '/stages/' + stage + '/simulations/' + simulationId,
        method: 'GET'
    };
    const headers = {};
    const meta = {
        apiName: 'get-simulation',
        callerCommand: callerCommand
    }
    requestWrapper.request(meta, CONSTANTS.SMAPI.VERSION.V2, general, headers, null, profile, doDebug, callback);
}

function callCreateSkill(manifest, vendorId, profile, doDebug, callback) {
    let general = {
        url: '/skills/',
        method: 'POST'
    };
    let headers = {};
    let payload = {
        vendorId: vendorId,
        manifest: manifest.manifest
    };
    requestWrapper.request('create-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

function callGetSkill(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/manifest';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function handleGetSkillRedirect(redirectUrl, profile, doDebug, callback) {
    let headers = {};
    requestWrapper.requestWithUrl('get-skill-status', redirectUrl, 'GET', headers, null, profile, doDebug, callback);
}

function callUpdateSkill(skillId, stage, manifest, etag, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/manifest';
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = etag ? {'If-Match': etag} : {};
    let payload = {
        manifest: manifest.manifest
    };
    requestWrapper.request('update-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

function callDeleteSkill(skillId, profile, doDebug, callback) {
    let url = '/skills/' + skillId;
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('delete-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callGetSkillStatus(skillId, resourcesList, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/status';
    // if resourcesList is null or empty, which means user doesn't specify certain resource,
    // SMAPI will return status for all the resources.
    if (resourcesList !== null && resourcesList.length !== 0) {
        url += ('?' + querystring.stringify({resource: resourcesList}));
    }

    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-skill-status', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callListSkills(vendorId, nextToken, numberOfReturnSkills, profile, doDebug, callback) {
    let url = nextToken ?
        '/skills?vendorId=' + vendorId + '&nextToken=' + nextToken + '&maxResults=' + (numberOfReturnSkills || 50) :
        '/skills?vendorId=' + vendorId + '&maxResults=' + (numberOfReturnSkills || 50);

    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('list-skills', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callGetModel(skillId, stage, locale, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/interactionModel/locales/' + locale;
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-model', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callGetUtteranceData(skillId, queryParameters, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/history/intentRequests?' + querystring.stringify(queryParameters);
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-utterance-data', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callHeadModel(skillId, stage, locale, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/interactionModel/locales/' + locale;
    let general = {
        url: url,
        method: 'HEAD'
    };
    let headers = {};
    requestWrapper.request('head-model', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callUpdateModel(skillId, stage, locale, modelSchema, eTag, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/interactionModel/locales/' + locale;
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = eTag ? { 'If-Match': eTag } : {};
    let payload = {
        interactionModel: modelSchema.interactionModel
    };
    requestWrapper.request('update-model', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

function callCreateAccountLinking(skillId, stage, accountLinking, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/accountLinkingClient';
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    let payload = {
        accountLinkingRequest: accountLinking
    };
    requestWrapper.request('create-account-linking', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

function callGetAccountLinking(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/accountLinkingClient';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-account-linking', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callDeleteAccountLinking(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/accountLinkingClient';
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('delete-account-linking', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callListVendor(profile, doDebug, callback) {
    let url = '/vendors';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('list-vendors', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

function callSubmit(skillId, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/submit';
    let general = {
        url: url,
        method: 'POST'
    };
    let headers = {};
    requestWrapper.request('submit', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callWithdraw(skillId, withdrawReason, withdrawMessage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/withdraw';
    let general = {
        url: url,
        method: 'POST'
    };
    let headers = {};
    let payload = {
        reason: withdrawReason,
        message: withdrawMessage
    };
    requestWrapper.request('withdraw', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

function callEnableSkill(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/enablement';
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    requestWrapper.request('enable-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callDisableSkill(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/enablement';
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('disable-skill', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callGetSkillEnablement(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/enablement';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-skill-enablement', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callGetSkillCredentials(skillId, profile, doDebug, callback) {
    const url = `/skills/${skillId}/credentials`;
    const general = {
        url: url,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('get-skill-credentials', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callAddPrivateDistributionAccount(skillId, stage, accountId, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/privateDistributionAccounts/' + accountId;
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    requestWrapper.request('add-private-distribution-account', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callDeletePrivateDistributionAccount(skillId, stage, accountId, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/privateDistributionAccounts/' + accountId;

    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('delete-private-distribution-account', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

function callListPrivateDistributionAccounts(skillId, stage, nextToken, numberOfReturnedAccounts, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/privateDistributionAccounts?';
    url += nextToken ?
        'nextToken=' + nextToken + '&maxResults=' + (numberOfReturnedAccounts || 50) :
        'maxResults=' + (numberOfReturnedAccounts || 50);
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('list-private-distribution-accounts', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);

}

/**
 * In-skill product APIs
 */

/**
 * Create in-skill product API wrapper
 * @param vendorId the vendor Id used to create in-skill product
 * @param ispDefinition [Required] in-skill product definition object
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callCreateISP(vendorId, ispDefinition, profile, doDebug, callback) {
    let errorList = [];
    if (!vendorId) {
        errorList.push(ispConstants.ERROR.CANNOT_GET_VENDOR_ID_FROM_PROFILE);
    }
    if (!ispDefinition) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_DEFINITION);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    let general = {
        url: ispConstants.URL.ISP_BASE,
        method: 'POST'
    };

    let headers = {};

    let payload = {
        vendorId: vendorId,
        inSkillProductDefinition: ispDefinition,
    };

    requestWrapper.request(ispConstants.API.CREATE_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * Update in-skill product API wrapper
 * @param ispId [Required] in-skill product ID
 * @param ispDefinition [Required] in-skill product definition in JSON
 * @param stage [Required] in-skill product stage
 * @param profile user profile
 * @param eTag ETag to define the in-skill product version
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callUpdateISP(ispId, ispDefinition, stage, profile, eTag, doDebug, callback) {
    let errorList = [];
    if (!ispId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!ispDefinition) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_DEFINITION);
    }
    if (!stage) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_STAGE);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    const url = ispConstants.URL.ISP_BASE + '/' + ispId + '/stages/' + stage;

    let general = {
        url: url,
        method: 'PUT'
    };

    let headers = {};

    if (eTag) {
        headers = {
            'If-Match': eTag
        };
    }

    let payload = {
        inSkillProductDefinition: ispDefinition
    };

    requestWrapper.request(ispConstants.API.UPDATE_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * Get the information of given in-skill product API wrapper
 * @param ispId [Required] in-skill product ID
 * @param stage [Required] in-skill product stage
 * @param getSummary summary mode
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callGetISP(ispId, stage, getSummary, profile, doDebug, callback) {
    let errorList = [];
    if (!ispId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!stage) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_STAGE);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    const url = ispConstants.URL.ISP_BASE + '/' + ispId + '/stages/' + stage + (getSummary ? '/summary' : '');

    let general = {
        url: url,
        method: 'GET'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.GET_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * Associate in-skill product with skill API wrapper
 * @param ispId [Required] in-skill product ID
 * @param skillId [Required] skill ID
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callAssociateISP(ispId, skillId, profile, doDebug, callback) {
    let errorList = [];
    if (!ispId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!skillId) {
        errorList.push(ispConstants.ERROR.MISSING_SKILL_ID);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    const url = ispConstants.URL.ISP_BASE + '/' + ispId + '/skills/' + skillId;

    let general = {
        url: url,
        method: 'PUT'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.ASSOCIATE_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * Disassociate a in-skill product from a skill API wrapper
 * @param ispId [Required] in-skill product ID
 * @param skillId [Required] skill ID
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callDisassociateISP(ispId, skillId, profile, doDebug, callback) {
    let errorList = [];
    if (!ispId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!skillId) {
        errorList.push((ispConstants.ERROR.MISSING_SKILL_ID));
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    const url = ispConstants.URL.ISP_BASE + '/' + ispId + '/skills/' + skillId;

    let general = {
        url: url,
        method: 'DELETE'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.DISASSOCIATE_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * List skills associated with given in-skill product API wrapper
 * @param ispId [Required] in-skill product ID
 * @param stage [Required] in-skill product stage
 * @param maxResults max number of results returned, default is 50
 * @param nextToken token to get next page of results
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callListSkillsForISP(ispId, stage, maxResults, nextToken, profile, doDebug, callback) {
    let errorList = [];
    if (!ispId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!stage) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_STAGE);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    let queryObject = {};

    queryObject.maxResults = maxResults ? maxResults : ispConstants.NUMBERS.DEFAULT_ISP_MAX_RESULTS;

    if (nextToken) {
        queryObject.nextToken = nextToken;
    }

    let qs = (maxResults || nextToken) ? querystring.stringify(queryObject) : '';

    let url = ispConstants.URL.ISP_BASE + '/' + ispId + '/stages/' + stage + '/skills?' + qs;

    let general = {
        url: url,
        method: 'GET'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.LIST_SKILLS_FOR_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * Delete in-skill product API wrapper
 * @param ispId [Required] in-skill product ID
 * @param stage [Required] in-skill product ID, only development stage supported
 * @param eTag Etag to define the in-skill product version
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callDeleteISP(ispId, stage, eTag, profile, doDebug, callback) {
    let errorList = [];
    if (!ispId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!stage) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_STAGE);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    const url = ispConstants.URL.ISP_BASE + '/' + ispId + '/stages/' + stage;

    let general = {
        url: url,
        method: 'DELETE'
    };

    let headers = {};

    if (eTag) {
        headers = {
            'If-Match': eTag
        };
    }

    requestWrapper.request(ispConstants.API.DELETE_ISP, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * List in-skill product associated with the given skill API wrapper
 * @param skillId [Required] skill ID
 * @param stage [Required] in-skill product stage
 * @param maxResults max number of results returned, default is 50
 * @param nextToken token to get next page of results
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callListISPForSkill(skillId, stage, maxResults, nextToken, profile, doDebug, callback) {
    let errorList = [];
    if (!skillId) {
        errorList.push(ispConstants.ERROR.MISSING_SKILL_ID);
    }
    if (!stage) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_STAGE);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    let queryObject = {};

    queryObject.maxResults = maxResults ? maxResults : ispConstants.NUMBERS.DEFAULT_ISP_MAX_RESULTS;

    if (nextToken) {
        queryObject.nextToken = nextToken;
    }

    let qs = (maxResults || nextToken) ? querystring.stringify(queryObject) : '';

    const url = ispConstants.URL.SKILL_BASE + '/' + skillId + '/stages/' + stage + '/inSkillProducts?' + qs;

    let general = {
        url: url,
        method: 'GET'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.LIST_ISP_FOR_SKILL, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * List in-skill product with the given vendor API wrapper
 * @param vendorId [Required] vendor ID for the in-skill product
 * @param productIdList list of in-skill product Ids
 * @param referenceName the reference name of the in-skill product
 * @param type in-skill product type
 * @param stage in-skill product stage
 * @param status in-skill product status
 * @param isAssociatedWithSkill is the in-skill product associated with any skill
 * @param maxResults max number of results returned, default is 50
 * @param nextToken token to get the next page of results
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callListISPForVendor(vendorId, productIdList, referenceName, type, stage, status, isAssociatedWithSkill, maxResults, nextToken, profile, doDebug, callback) {
    if (!vendorId) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: [ispConstants.ERROR.MISSING_VENDOR_ID]
        });
        return;
    }

    let queryObject = {};

    queryObject.vendorId = vendorId;

    if (productIdList && productIdList.length > 0) {
        queryObject.productId = productIdList.filter(id => id);
    }

    if (referenceName) {
        queryObject.referenceName = referenceName;
    }

    if (type) {
        queryObject.type = type;
    }

    if (stage) {
        queryObject.stage = stage;
    }

    if (status) {
        queryObject.status = status;
    }

    if (isAssociatedWithSkill) {
        queryObject.isAssociatedWithSkill = isAssociatedWithSkill;
    }

    if (maxResults) {
        queryObject.maxResults = maxResults;
    }

    if (nextToken) {
        queryObject.nextToken = nextToken;
    }

    let qs = querystring.stringify(queryObject);

    const url = ispConstants.URL.ISP_BASE + '?' + qs;

    let general = {
        url: url,
        method: 'GET'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.LIST_ISP_FOR_VENDOR, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * Reset the in-skill-product entitlement
 * @param productId [Required] the product id of in-skill product
 * @param stage [Required] the stage of in-skill product
 * @param profile the user profile
 * @param doDebug ASK CLI debug mode
 * @param callback the call back to handle the response
 */
function callResetISPEntitlement(productId, stage, profile, doDebug, callback) {
    let errorList = [];
    if (!productId) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_PRODUCT_ID);
    }
    if (!stage) {
        errorList.push(ispConstants.ERROR.MISSING_ISP_STAGE);
    }
    if (errorList.length > 0) {
        callback({
            message: ispConstants.ERROR.INVALID_API_PARAMETERS,
            violations: errorList
        });
        return;
    }

    const url = ispConstants.URL.ISP_BASE + '/' + productId + '/stages/' + stage + '/entitlement';

    let general = {
        url: url,
        method: 'DELETE'
    };

    let headers = {};

    requestWrapper.request(ispConstants.API.RESET_ISP_ENTITLEMENT, CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, (apiName, response) => {
        ispUtil.handleAPIResponse(apiName, response, callback);
    });
}

/**
 * List catalogs by vendorId
 * @param {string} catalogId 
 * @param {number} maxResults 
 * @param {string} nextToken 
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callListCatalogsByVendor(vendorId, maxResults, nextToken, profile, doDebug, callback) {
    const qsObject = {
        vendorId: vendorId
    };
    if (maxResults) {
        qsObject.maxResults = maxResults;
    }
    if (nextToken) {
        qsObject.nextToken = nextToken;
    }
    let qs = querystring.stringify(qsObject);
    qs = qs ? `?${qs}` : '';

    const general = {
        url: `/catalogs${qs}`,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('list-catalogs-by-vendor', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

/**
 * Create a catalog
 * @param catalogType [required] type of the catalog
 * @param catalogTitle [required] title of the catalog
 * @param catalogUsage [required] usage of the catalog
 * @param vendorId vendor Id associate with the developer account
 * @param profile the user profile
 * @param doDebug ASK CLI debug mode
 * @param callback the call back to handle the response
 */
function callCreateCatalog(catalogType, catalogTitle, catalogUsage, vendorId, profile, doDebug, callback) {
    const general = {
        url: '/catalogs/',
        method: 'POST'
    };
    const headers = {};
    const payload = {
        vendorId: vendorId,
        type: catalogType,
        title: catalogTitle,
        usage: catalogUsage
    };
    requestWrapper.request('create-catalog', CONSTANTS.SMAPI.VERSION.V0, general, headers, payload, profile, doDebug, callback);
}

/**
 * Get catalog by catalogId
 * @param {string} catalogId 
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callGetCatalog(catalogId, profile, doDebug, callback) {
    const general = {
        url: `/catalogs/${catalogId}`,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('get-catalog', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

/**
 * List uploads by catalogId
 * @param {string} catalogId 
 * @param {number} maxResults 
 * @param {string} nextToken 
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callListCatalogUploads(catalogId, maxResults, nextToken, profile, doDebug, callback) {
    const qsObject = {};
    if (maxResults) {
        qsObject.maxResults = maxResults;
    }
    if (nextToken) {
        qsObject.nextToken = nextToken;
    }
    let qs = querystring.stringify(qsObject);
    qs = qs ? `?${qs}` : '';

    const general = {
        url: `/catalogs/${catalogId}/uploads${qs}`,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('list-catalog-uploads', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

/**
 * Create an upload for the catalog
 * @param {string} catalogId 
 * @param {number} partsNumber
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callCreateCatalogUpload(catalogId, partsNumber, profile, doDebug, callback) {
    const general = {
        url: `/catalogs/${catalogId}/uploads`,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        numberOfUploadParts: partsNumber
    };
    requestWrapper.request('create-catalog-upload', CONSTANTS.SMAPI.VERSION.V0, general, headers, payload, profile, doDebug, callback);
}

/**
 * Get catalog upload
 * @param {string} catalogId 
 * @param {string} uploadId 
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callGetCatalogUpload(catalogId, uploadId, profile, doDebug, callback) {
    const general = {
        url: `/catalogs/${catalogId}/uploads/${uploadId}`,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('get-catalog-upload', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

/**
 * Complete an upload for the catalog
 * @param {string} catalogId 
 * @param {string} uploadId 
 * @param {Array} partETagsList
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callCompleteCatalogUpload(catalogId, uploadId, partETagsList, profile, doDebug, callback) {
    const general = {
        url: `/catalogs/${catalogId}/uploads/${uploadId}`,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        partETags: partETagsList
    };
    requestWrapper.request('complete-catalog-upload', CONSTANTS.SMAPI.VERSION.V0, general, headers, payload, profile, doDebug, callback);
}

/**
 * List catalogs by skillId
 * @param {string} catalogId 
 * @param {number} maxResults 
 * @param {string} nextToken 
 * @param {string} profile 
 * @param {boolean} doDebug 
 * @param {function} callback 
 */
function callListCatalogsBySkill(skillId, maxResults, nextToken, profile, doDebug, callback) {
    const qsObject = {};
    if (maxResults) {
        qsObject.maxResults = maxResults;
    }
    if (nextToken) {
        qsObject.nextToken = nextToken;
    }
    let qs = querystring.stringify(qsObject);
    qs = qs ? `?${qs}` : '';

    const general = {
        url: `/skills/${skillId}/catalogs${qs}`,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('list-catalogs-by-skill', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

function callAssociateCatalogWithSkill(skillId, catalogId, profile, doDebug, callback) {
    const general = {
        url: `/skills/${skillId}/catalogs/${catalogId}`,
        method: 'PUT'
    };
    const headers = {};
    requestWrapper.request('associate-catalog-with-skill', CONSTANTS.SMAPI.VERSION.V0, general, headers, null, profile, doDebug, callback);
}

/**
 * Get the beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callGetBetaTest(skillId, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest';
    const general = {
        url: url,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('get-beta-test', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

/**
 * Create a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param feedbackEmail email address for providing feedback
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callCreateBetaTest(skillId, feedbackEmail, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        feedbackEmail: feedbackEmail
    };
    requestWrapper.request('create-beta-test', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

/**
 * Update the beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param feedbackEmail email address for providing feedback
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callUpdateBetaTest(skillId, feedbackEmail, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest';
    const general = {
        url: url,
        method: 'PUT'
    };
    const headers = {};
    const payload = {
        feedbackEmail: feedbackEmail
    };
    requestWrapper.request('update-beta-test', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

/**
 * Start a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callStartBetaTest(skillId, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest/start';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    requestWrapper.request('start-beta-test', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

/**
 * End a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callEndBetaTest(skillId, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest/end';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    requestWrapper.request('end-beta-test', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

/**
 * List testers in a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param nextToken token to get next page of results
 * @param maxResults max number of results returned, default is 50
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callListBetaTesters(skillId, nextToken, maxResults, profile, doDebug, callback) {
    const url = nextToken ?
        '/skills/' + skillId + '/betaTest/testers?nextToken=' + nextToken + '&maxResults=' + (maxResults || CONSTANTS.BETA_TEST.DEFAULT_LIST_MAX_RESULT) :
        '/skills/' + skillId + '/betaTest/testers?maxResults=' + (maxResults || CONSTANTS.BETA_TEST.DEFAULT_LIST_MAX_RESULT) ;
    const general = {
        url: url,
        method: 'GET'
    };
    const headers = {};
    requestWrapper.request('list-beta-testers', CONSTANTS.SMAPI.VERSION.V1, general, headers, null, profile, doDebug, callback);
}

/**
 * Add testers to a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param testers tester email id list
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callAddBetaTesters(skillId, testers, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest/testers/add';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        testers: testers
    };
    requestWrapper.request('add-beta-testers', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

/**
 * Remove testers from a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param testers tester email id list
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callRemoveBetaTesters(skillId, testers, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest/testers/remove';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        testers: testers
    };
    requestWrapper.request('remove-beta-testers', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

/**
 * Send reminder to testers in a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param testers tester email id list
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callSendReminderToBetaTesters(skillId, testers, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest/testers/sendReminder';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        testers: testers
    };
    requestWrapper.request('send-reminder-to-beta-testers', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

/**
 * Request feedback from testers in a beta test of given Alexa skill
 * @param skillId [Required] skill id
 * @param testers tester email id list
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callRequestFeedbackFromBetaTesters(skillId, testers, profile, doDebug, callback) {
    const url = '/skills/' + skillId + '/betaTest/testers/requestFeedback';
    const general = {
        url: url,
        method: 'POST'
    };
    const headers = {};
    const payload = {
        testers: testers
    };
    requestWrapper.request('request-feedback-from-beta-testers', CONSTANTS.SMAPI.VERSION.V1, general, headers, payload, profile, doDebug, callback);
}

/**
 * Profile utterance with NLU
 * @param skillId [Required] skill id
 * @param stage skill stage, default is development
 * @param locale [Required] skill locale
 * @param utterance [Required] utterance to be profiled
 * @param multiTurnToken multiturn token for dialog
 * @param profile user profile
 * @param doDebug CLI debug mode
 * @param callback callback function from command
 */
function callProfileNlu(skillId, stage, locale, utterance, multiTurnToken, profile, doDebug, callback) {
     const skillStage = stage || CONSTANTS.SKILL.STAGE.DEVELOPMENT;
     const url = '/skills/' + skillId + '/stages/' + skillStage + '/interactionModel/locales/' + locale + '/profileNlu';
     const general = {
         url: url,
         method: 'POST'
     };
     const headers = {};
     const payload = {
        utterance: utterance
     };

     if (multiTurnToken) {
         payload.multiTurnToken = multiTurnToken;
     }

     requestWrapper.request('nlu-profile', CONSTANTS.SMAPI.VERSION.V1, general, headers,
             payload, profile, doDebug, callback);
}