'use strict';

const fs = require('fs');
const inquirer = require('inquirer');
const prettyBytes = require('pretty-bytes');

const apiWrapper = require('./api-wrapper');
const S3Client = require('../clients/s3-client');
const tools = require('../utils/tools');
const profileHelper = require('../utils/profile-helper');
const CONSTANTS = require('../utils/constants');

// Public
module.exports = {
    createCommand: (commander) => {
        buildListCatalogsCommand(commander);
        buildGetCatalogCommand(commander);
        buildCreateCatalogCommand(commander);
        buildListCatalogUploadsCommand(commander);
        buildGetCatalogUploadCommand(commander);
        buildAssociateCatalogCommand(commander);

        buildUploadCatalogCommand(commander);
    }
};

function buildListCatalogsCommand(commander) {
    commander
        .command('list-catalogs')
        .usage('[--max-results <max-results>] [--next-token <next-token>] [-p|--profile <profile>] [--debug]')
        .description('get the list of catalogs by the profile vendor')
        .option('--max-results <max-results>', 'max results returned by the request')
        .option('--next-token <next-token>', 'next token if the result is paginated')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find the vendor-id associated with the current profile.');
            process.exit(1);
        }
        apiWrapper.callListCatalogsByVendor(vendorId, options.maxResults, options.nextToken, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildCreateCatalogCommand(commander) {
    commander
        .command('create-catalog')
        .usage('<--catalog-type <catalog-type>> <--catalog-title <catalog-title>> <--catalog-usage <catalog-usage>> [-p|--profile <profile>] [--debug]')
        .description('create a catalog')
        .option('--catalog-type <catalog-type>', 'type of the catalog')
        .option('--catalog-title <catalog-title>', 'title of the catalog')
        .option('--catalog-usage <catalog-usage>', 'usage of the catalog')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        if (!options.catalogType) {
            console.warn('Please input required parameter: catalog-type.');
            process.exit(1);
        }
        if (!options.catalogTitle) {
            console.warn('Please input required parameter: catalog-title.');
            process.exit(1);
        }
        if (!options.catalogUsage) {
            console.warn('Please input required parameter: catalog-usage.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        const vendorId = profileHelper.resolveVendorId(profile);
        if (!vendorId) {
            console.error('[Error]: cannot find vendor-id associated with the current profile.');
            process.exit(1);
        }

        apiWrapper.callCreateCatalog(options.catalogType, options.catalogTitle, options.catalogUsage, vendorId, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            console.log(`Catalog created successfully. Catalog ID: ${response.id}`);
        });
    }
}

function buildGetCatalogCommand(commander) {
    commander
        .command('get-catalog')
        .usage('<-c|--catalog-id <catalog-id>> [-p|--profile <profile>] [--debug]')
        .description('get the catalog by catalog-id')
        .option('-c, --catalog-id <catalog-id>', 'id of the catalog')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        if (!options.catalogId) {
            console.warn('Please input required parameter: catalog-id.');
            process.exit(1);
        }

        const profile = profileHelper.runtimeProfile(options.profile);
        apiWrapper.callGetCatalog(options.catalogId, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildListCatalogUploadsCommand(commander) {
    commander
        .command('list-catalog-uploads')
        .usage('<-c|--catalog-id <catalog-id>> [--max-results <max-results>] [--next-token <next-token>] [-p|--profile <profile>] [--debug]')
        .description('list all the catalogs by catalog-id')
        .option('-c, --catalog-id <catalog-id>', 'id of the catalog')
        .option('--max-results <max-results>', 'max results returned by the request')
        .option('--next-token <next-token>', 'next token if the result is paginated')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        if (!options.catalogId) {
            console.warn('Please input required parameter: catalog-id.');
            process.exit(1);
        }

        const profile = profileHelper.runtimeProfile(options.profile);
        apiWrapper.callListCatalogUploads(options.catalogId, options.maxResults, options.nextToken, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildGetCatalogUploadCommand(commander) {
    commander
        .command('get-catalog-upload')
        .usage('<-c|--catalog-id <catalog-id>> <-u|--upload-id <upload-id>> [-p|--profile <profile>] [--debug]')
        .description('get the catalog upload by catalog-id and upload-id')
        .option('-c, --catalog-id <catalog-id>', 'id of the catalog')
        .option('-u, --upload-id <upload-id>', 'id of the catalog upload')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        if (!options.catalogId) {
            console.warn('Please input required parameter: catalog-id.');
            process.exit(1);
        }
        if (!options.uploadId) {
            console.warn('Please input required parameter: upload-id.');
            process.exit(1);
        }

        const profile = profileHelper.runtimeProfile(options.profile);
        apiWrapper.callGetCatalogUpload(options.catalogId, options.uploadId, profile, options.debug, (data) => {
            const response = tools.convertDataToJsonObject(data.body);
            if (response) {
                console.log(JSON.stringify(response, null, 2));
            }
        });
    }
}

function buildAssociateCatalogCommand(commander) {
    commander
        .command('associate-catalog-with-skill')
        .usage('<-s|--skill-id <skill-id>> <-c|--catalog-id <catalog-id>> [-p|--profile <profile>] [--debug]')
        .description('associate a catalog with a skill')
        .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
        .option('-c, --catalog-id <catalog-id>', 'catalog-id of the catalog')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        if (!options.skillId) {
            console.warn('Please input required parameter: skill-id.');
            process.exit(1);
        }
        if (!options.catalogId) {
            console.warn('Please input required parameter: catalog-id.');
            process.exit(1);
        }
        const profile = profileHelper.runtimeProfile(options.profile);
        apiWrapper.callAssociateCatalogWithSkill(options.skillId, options.catalogId, profile, options.debug, (data) => {
            console.log('Catalog associated with the skill successfully.');
        });
    }
}

function buildUploadCatalogCommand(commander) {
    commander
        .command('upload-catalog')
        .usage('<-c|--catalog-id <catalog-id>> <-f|--file <file>> [-p|--profile <profile>] [--debug]')
        .description('upload a file for the catalog')
        .option('-c, --catalog-id <catalog-id>', 'catalog-id of a catalog')
        .option('-f, --file <file>', 'the file to upload to the catalog')
        .option('-p, --profile <profile>', 'ask cli profile')
        .option('--debug', 'ask cli debug mode')
        .action(handle);

    function handle(options) {
        if (typeof options === 'string') {
            console.error('Please run the command with -h for usage instruction. Make sure you use options or option-value pair for inputs.');
            process.exit(1);
        }
        if (!options.catalogId) {
            console.warn('Please input required parameter: catalog-id.');
            process.exit(1);
        }
        if (!options.file) {
            console.warn('Please input required parameter: file.');
            process.exit(1);
        }
        if (!fs.existsSync(options.file)) {
            console.error('[Error]: Please make sure the file to upload to the catalog exists.');
            process.exit(1);
        }

        const profile = profileHelper.runtimeProfile(options.profile);
        _multiPartUploadCatalog(options.catalogId, options.file, profile, options.debug, (err) => {
            if (err) {
                console.error(err);
                process.exit(1);
            }
            console.log('Catalog uploaded successfully.');
        });
    }
}

function _multiPartUploadCatalog(catalogId, filePath, profile, doDebug, callback) {
    const { totalSize, calculatedPartSize, calculatedPartsNumber } = _resolvePartSize(filePath);
    if (totalSize === 0 || calculatedPartSize === 0 || calculatedPartsNumber === 0) {
        return callback('[Error]: The file to be uploaded cannot be empty.');
    }

    _confirmOrOverwritePartSize(totalSize, calculatedPartSize, calculatedPartsNumber, (partSize, partsNumber) => {
        apiWrapper.callCreateCatalogUpload(catalogId, partsNumber, profile, doDebug, (data) => {
            const createResponse = tools.convertDataToJsonObject(data.body);
            if (!createResponse) {
                return process.nextTick(() => {
                    callback('[Error]: The response from create-catalog-upload should not be empty.');
                });
            }
            if (!createResponse.id || !createResponse.presignedUploadParts) {
                return callback('[Error]: The response of create-catalog-upload is not valid.');
            }

            const uploadId = createResponse.id;
            const uploadPartsMap = _transformUploadArrayToMap(createResponse.presignedUploadParts);
            if (!uploadPartsMap) {
                return callback('[Error]: The response from create-catalog-upload is empty.');
            }
            
            console.log(`Upload (upload-id: ${uploadId}) created successfully. Upload starts...`);
            S3Client.multipartsUploadToPresignedUrls(uploadPartsMap, filePath, totalSize, partSize, partsNumber, (err, partETagsList) => {
                if (err) {
                    return callback(err);
                }
                apiWrapper.callCompleteCatalogUpload(catalogId, uploadId, partETagsList, profile, doDebug, () => {
                    callback();
                });
            });
        });
    });
}

function _resolvePartSize(file) {
    const fileSize = fs.statSync(file).size;
    const minAverageSize = Math.max(Math.ceil(fileSize / CONSTANTS.CONFIGURATION.S3.MULTIPART_UPLOAD.MAX_PART_COUNT),
                                    CONSTANTS.CONFIGURATION.S3.MULTIPART_UPLOAD.DEFAULT_PART_SIZE);
    const partsNumber = Math.ceil(fileSize / minAverageSize);

    return {
        totalSize: fileSize,
        calculatedPartSize: minAverageSize,
        calculatedPartsNumber: partsNumber
    };
}

function _confirmOrOverwritePartSize(totalSize, partSize, partsNumber, callback) {
    if (partsNumber === 1) {
        return process.nextTick(() => {
            callback(partSize, partsNumber);
        });
    }

    inquirer.prompt({
        type: 'confirm',
        name: 'isAllowed',
        message: `CLI's about to partition this ${prettyBytes(totalSize)} file to ${partsNumber} parts and upload in parallel.\n  Do you agree with this partition?`
    }).then((answer) => {
        if (answer.isAllowed) {
            return callback(partSize, partsNumber);
        }

        const maxPartitions = Math.floor(totalSize / CONSTANTS.CONFIGURATION.S3.MULTIPART_UPLOAD.MIN_PART_SIZE);
        inquirer.prompt({
            type: 'input',
            name: 'preferredPartsNumber',
            message: `Please tell us your preferred number of partitions (1 ~ ${maxPartitions}):`,
            validate: (input) => {
                if (!Number(input) || Number(input) <= 0 || Number(input) > maxPartitions) {
                    return `Parts number should be a valid integer in the range (1 ~ ${maxPartitions}) to make sure each part is larger then ${CONSTANTS.CONFIGURATION.S3.MULTIPART_UPLOAD.MIN_PART_SIZE_DISPLAY}.`;
                }
                return true;
            }
        }).then((answer) => {
            const preferredParts = Number(answer.preferredPartsNumber);
            callback(Math.ceil(totalSize / preferredParts), preferredParts);
        });
    });
}

function _transformUploadArrayToMap(uploadPartsArray) {
    if (!uploadPartsArray || uploadPartsArray.length === 0) {
        return null;
    }
    
    const result = new Map();
    uploadPartsArray.forEach(cell => {
        result.set(cell.partNumber, cell.url);
    });
    return result;
}
