'use strict';

const fs = require('fs');
const request = require('request');
const prettyBytes = require('pretty-bytes');

const ParallelStream = require('../utils/stream-utility').ParallelStream;
const displayUtility = require('../utils/display-utility');
const CONSTANTS = require('../utils/constants');

/**
 * Class for S3 client
 */
module.exports = class S3Client {
    
    /**
     * Upload to a pre-signed S3 url with custom options
     * @param {string} url 
     * @param {object} options 
     * @param {function} callback 
     */
    static presignedPutObject(url, options, callback) {
        if (typeof url !== 'string' || !url.trim()) {
            process.nextTick(() => {
                return callback('[Error]: The url for the S3 presigned URL upload must not be blank.');
            });
        }
        request.put(url, options, (err, response) => {
            callback(err, response);
        });
    }

    /**
     * Implement the multipart upload to S3. Run upload in parallel for each part by using ParallelStream.
     * 
     * @param {*} uploadPartsMap Mapping between partNumber and its eTag
     * @param {*} filePath The file to upload
     * @param {*} totalSize Total size of the file
     * @param {*} partSize The size of each partition (the last part can be smaller then this)
     * @param {*} partsNumber The number of partitions of the file
     * @param {*} callback 
     */
    static multipartsUploadToPresignedUrls(uploadPartsMap, filePath, totalSize, partSize, partsNumber, callback) {
        const progressSpinner = new displayUtility.Spinner();

        // Declarations of the state signals for the unordered stream upload 
        const partETagsList = [];
        let partBuffer = [];
        let partBufferTotalLength = 0;
        let loadedPartCount = 0;
        let loadedSize = 0;
        let uploadedPartCount = 0;
        let uploadedSize = 0;
    
        const uploadTask = (chunk, enc, done, push) => {
            // 1. Clear the partBuffer and append the rest (by slicing a partSize buffer) to the partBuffer
            const aggregatedBuffer = Buffer.concat(partBuffer);
            const rest = aggregatedBuffer.slice(partSize);
            partBuffer = [];
            partBuffer.push(rest);
            partBufferTotalLength = rest.length;
            
            // 2. Upload by S3 presigned URL
            const threadPartNum = ++loadedPartCount; // Need to store the part number for current "thread" to avoid wrong mapping for eTag and partNumber
            progressSpinner.message(` ${loadedPartCount}/${partsNumber} uploads have been requested. Currently ${uploadedPartCount}/${partsNumber} parts and ${prettyBytes(uploadedSize)}/${prettyBytes(totalSize)} have been uploaded...`);
            S3Client.presignedPutObject(
                uploadPartsMap.get(threadPartNum),
                { body: aggregatedBuffer },
                (err, response) => {
                    if (err) {
                        return done(err);
                    }

                    // 3. Update uploaded parts state.
                    uploadedPartCount++;
                    uploadedSize += aggregatedBuffer.byteLength;
                    partETagsList.push({
                        eTag: response.headers.etag,
                        partNumber: threadPartNum
                    });

                    progressSpinner.message(` ${loadedPartCount}/${partsNumber} uploads have been requested. Currently ${uploadedPartCount}/${partsNumber} parts and ${prettyBytes(uploadedSize)}/${prettyBytes(totalSize)} have been uploaded...`);
                    done();
                }
            );
        };

        const processStreamBuffer = (chunk, enc) => {
            // Process data whenever stream buffer is readable. Accumulate part and total length for the partBuffer
            partBuffer.push(chunk);
            partBufferTotalLength += chunk.byteLength;
            loadedSize += chunk.byteLength;
        };

        const verifyAndFinish = () => {
            progressSpinner.stop();
            if (partETagsList.length !== partsNumber) {
                return callback(`[Error]: Multipart upload failed. There are ${partsNumber - partETagsList.length} part(s) over ${partsNumber} parts not uploaded. Please try again!`);
            }
            for (let element of partETagsList) {
                const tmpETag = element.eTag;
                if (typeof tmpETag !== 'string' || !tmpETag.trim()) {
                    return callback(`[Error]: Failed to get the ETag for part number ${element.partNumber}. Please try again!`);
                }
            }
            callback(null, partETagsList);
        };
    
        const parallelStreamOptions = {
            willTransform: () => {
                return partBufferTotalLength >= partSize || (loadedPartCount === partsNumber - 1 && loadedSize === totalSize);
            },
            concurrency: CONSTANTS.CONFIGURATION.S3.MULTIPART_UPLOAD.CONCURRENCY,
            isObjectMode: false,
        };
        
        // Main stream processing
        progressSpinner.start();
        fs.createReadStream(filePath)
          .pipe(new ParallelStream(uploadTask, processStreamBuffer, verifyAndFinish, parallelStreamOptions))
          .on('error', (err) => {
              callback(err);
          });
    }
};
