const awsProfileHandler = require('aws-profile-handler');
const path = require('path');
const fs = require('fs');
const os = require('os');
const mkdirp = require('mkdirp');
const CONSTANTS = require('@src/utils/constants');
const profileHelper = require('@src/utils/profile-helper');
const ui = require('./ui');
const messages = require('./messages');

module.exports = {
    setup,
    resolve
};

function resolve(askProfile, callback) {
    const awsPath = path.join(os.homedir(), '.aws', 'credentials');
    try {
        const awsProfileList = fs.existsSync(awsPath) ? awsProfileHandler.listProfiles(awsPath) : [];
        ui.confirmSettingAws((setupChoice) => {
            if (!setupChoice) {
                console.log(messages.SKIP_AWS_INITIALIZATION);
                callback();
                return;
            }
            const envVarHandler = hasEnvironmentVariables() ? hasEnvVarHandler : noEnvVarHandler;
            envVarHandler(askProfile, awsProfileList, callback);
        });
    } catch (error) {
        callback(error);
    }
}

function hasEnvVarHandler(askProfile, awsProfileList, callback) {
    ui.selectEnvironmentVariables((selectEnvVarChoice) => {
        if (selectEnvVarChoice === 'No') {
            noEnvVarHandler(askProfile, awsProfileList, callback);
        } else {
            profileHelper.setupProfile(CONSTANTS.PLACEHOLDER.ENVIRONMENT_VAR.AWS_CREDENTIALS, askProfile);
            callback(null, CONSTANTS.PLACEHOLDER.ENVIRONMENT_VAR.AWS_CREDENTIALS);
        }
    });
}

function noEnvVarHandler(askProfile, awsProfileList, callback) {
    if (awsProfileList.length === 0) {
        setupAwsProfile(askProfile, callback);
    } else {
        ui.createNewOrSelectAWSProfile(awsProfileList, (awsProfile) => {
            if (awsProfile === 'Create new profile') {
                setupAwsProfile(askProfile, callback);
            } else {
                profileHelper.setupProfile(awsProfile, askProfile);
                callback(null, awsProfile);
            }
        });
    }
}

function setupAwsProfile(askProfile, callback) {
    setup((error, awsProfile) => {
        if (error) {
            callback(error);
            return;
        }
        profileHelper.setupProfile(awsProfile, askProfile);
        console.log(`AWS profile "${awsProfile}" was successfully created. The details are recorded in aws credentials file ($HOME/.aws/credentials).`);
        callback(null, awsProfile);
    });
}

function setup(callback) {
    // create credentials file if it doesn't exist
    const awsCredentialsDir = path.join(os.homedir(), '.aws');
    const awsCredentialsFile = path.join(awsCredentialsDir, 'credentials');
    if (!fs.existsSync(awsCredentialsFile)) {
        mkdirp.sync(awsCredentialsDir);
        fs.closeSync(fs.openSync(awsCredentialsFile, 'a'));
    }
    _setupCredentials(callback);
}

function _setupCredentials(callback) {
    let profileName = null;
    try {
        // if the credential file is empty, CLI will name the first profile as 'ask_cli_default'
        if (awsProfileHandler.listProfiles().length === 0) {
            profileName = CONSTANTS.COMMAND.INIT.AWS_DEFAULT_PROFILE_NAME;
        }
    } catch (error) {
        callback(error.message);
        return;
    }

    _askCredentialName(profileName, (name) => {
        if (awsProfileHandler.getProfileCredentials(name) === null) {
            ui.addNewCredentials((credentialObject) => {
                awsProfileHandler.addProfile(name, credentialObject);
                callback(null, name);
            });
        } else {
            ui.confirmOverwritingProfile(name, (confirm) => {
                if (confirm) {
                    ui.addNewCredentials((credentialObject) => {
                        awsProfileHandler.addProfile(name, credentialObject);
                        callback(null, name);
                    });
                } else {
                    // if the user chose not to overwrite the profile
                    // CLI will first list all the existing profile
                    // then call the set up function again to ask the user for a new profile name
                    console.log('List of the existing AWS profiles:');
                    awsProfileHandler.listProfiles().forEach(profile => console.log(`"${profile}"`));
                    _setupCredentials(callback);
                }
            });
        }
    });
}

function _askCredentialName(profileName, callback) {
    if (profileName) {
        process.nextTick(() => {
            callback(profileName);
        });
    } else {
        ui.requestAwsProfileName(callback);
    }
}

function hasEnvironmentVariables() {
    const accessKeyID = process.env.AWS_ACCESS_KEY_ID;
    const accessSecretKey = process.env.AWS_SECRET_ACCESS_KEY;
    return typeof accessKeyID === 'string' && accessKeyID.trim() && typeof accessSecretKey === 'string' && accessSecretKey.trim();
}
