const path = require('path');
const jsonfile = require('jsonfile');
const profileHelper = require('@src/utils/profile-helper');
const skillParser = require('@src/utils/skill-parser');
const CONSTANTS = require('@src/utils/constants');
const urlUtils = require('@src/utils/url-utils');
const HookManager = require('@src/utils/hook-manager');
const jsonUtility = require('@src/utils/json-utility');
const tools = require('@src/utils/tools');
const ui = require('./ui');
const helper = require('./helper');

module.exports = {
    handleOptions,
    changeSkillName,
    adaptToInputProfileName,
    changeLambdaName
};

function handleOptions(options) {
    if (typeof options === 'string') {
        console.error('[Error]: Invalid command. Please run "ask new -h" for help.');
        process.exit(1);
        return;
    }

    if (options.template && !options.url) {
        console.error('[Error]: The template option only works when the templates link is specified in the url option.');
        process.exit(1);
        return;
    }
    const profile = profileHelper.runtimeProfile(options.profile);
    _commandExecute(options, (error, skillNameResult) => {
        if (error && !skillNameResult) {
            console.error(error);
            process.exit(1);
            return;
        }
        const { userSkillName, filteredSkillName } = skillNameResult;
        console.log(`Skill "${userSkillName}" has been created based on the chosen template`);
        const cloneDir = path.join(process.cwd(), filteredSkillName);
        const skillFilePath = path.join(cloneDir, 'skill.json');
        const configPath = path.join(cloneDir, '.ask/config');
        tools.v1ManifestTransformer(skillFilePath, false, profile);
        /* TODO further functions needs to be refactored */
        module.exports.changeSkillName(userSkillName, skillFilePath);
        module.exports.adaptToInputProfileName(profile, configPath);
        module.exports.changeLambdaName(filteredSkillName, skillFilePath, options.lambdaName, profile, configPath);
        _postCommand(filteredSkillName, options.debug, profile, (err) => {
            if (err) {
                console.error(err);
                process.exit(1);
            }
        });
    });
}

// Private
function _commandExecute(options, callback) {
    // Command execution
    const isGitUrl = urlUtils.isUrlWithGitExtension(options.url);
    const isJsonUrl = urlUtils.isUrlWithJsonExtension(options.url);
    if (options.url && options.template) {
        if (!isGitUrl && !isJsonUrl) {
            callback('[Error]: Invalid url. Provide provide a valid git/json url.', null);
        }
        if (isGitUrl) {
            callback('[Error]: Not allowed to specify the template name with the url pointing to a git repository. '
            + 'Please either use only url option with the link pointing to a git repository, '
            + 'or use templates link url along with the target template name.', null);
            return;
        }
        _cloneFromIndexUrlAndTemplate(options, callback);
    } else if (options.url && !options.template) {
        if (isGitUrl) {
            _cloneFromGitUrl(options, callback);
        } else if (isJsonUrl) {
            _cloneFromIndexUrl(options, callback);
        } else {
            callback('[Error]: Invalid url. Provide provide a valid git/json url. ', null);
        }
    } else {
        _newWithOfficialTemplates(options, callback);
    }
}

function _cloneFromIndexUrlAndTemplate(options, callback) {
    helper.getTemplateMap(options.url, options.debug, (err, templateMap) => {
        if (err) {
            callback(err, null);
            return;
        }
        if (!templateMap) {
            callback(`[Error]: Cannot retrieve the template list at ${options.url}.`, null);
            return;
        }
        const templateInfo = templateMap[options.template];
        if (!templateInfo) {
            callback(`[Error]: The template ${options.template} not found in given url. Please provide a valid template name.`, null);
            return;
        }
        if (!templateInfo.gitUrl) {
            callback('[Error]: Please make sure your template name has "gitUrl" field with the url pointing to the git repository.', null);
            return;
        }
        const cloneUrl = templateInfo.gitUrl;
        ui.getSkillName(options.skillName, options.url, options.template, (skillNameError, userSkillName) => {
            if (skillNameError) {
                callback(skillNameError, null);
                return;
            }
            const filteredSkillName = skillParser.filterUserSkillName(userSkillName);
            helper.gitCloneTemplate(cloneUrl, filteredSkillName, (cloneErr) => {
                callback(cloneErr, cloneErr ? null : { userSkillName, filteredSkillName });
            });
        });
    });
}

function _cloneFromGitUrl(options, callback) {
    ui.getSkillName(options.skillName, options.url, options.template, (err, userSkillName) => {
        if (err) {
            callback(err, null);
            return;
        }
        const filteredSkillName = skillParser.filterUserSkillName(userSkillName);
        helper.gitCloneTemplate(options.url, filteredSkillName, (cloneError) => {
            callback(cloneError, cloneError ? null : { userSkillName, filteredSkillName });
        });
    });
}

function _cloneFromIndexUrl(options, callback) {
    if (!urlUtils.isUrlWithJsonExtension(options.url)) {
        callback('[Error]: Please provide a valid json url.', null);
        return;
    }
    _cloneRepo(options.url, options.skillName, options.template, options.debug, callback);
}

function _newWithOfficialTemplates(options, callback) {
    ui.getRunTime((err, runTime) => {
        if (err) {
            callback(err, null);
            return;
        }
        const url = CONSTANTS.LAMBDA.RUNTIME[runTime];
        _cloneRepo(url, options.skillName, options.template, options.debug, callback);
    });
}

function _cloneRepo(url, skillName, templateName, doDebug, callback) {
    helper.getTemplateMap(url, doDebug, (err, templateMap) => {
        if (err) {
            callback(err, null);
            return;
        }
        if (!templateMap) {
            callback(`[Error]: Cannot retrieve the template list at ${url}.`, null);
            return;
        }
        ui.getTargetTemplateName(Object.keys(templateMap), null, (getTemplateError, templateInfo) => {
            if (getTemplateError) {
                callback(getTemplateError, null);
                return;
            }
            const cloneUrl = templateMap[templateInfo].gitUrl;
            if (!cloneUrl) {
                callback(`[Error]: gitUrl for ${templateInfo} is not present in template map`);
                return;
            }
            ui.getSkillName(skillName, cloneUrl, templateName, (getskillNameError, userSkillName) => {
                if (getskillNameError) {
                    callback(getskillNameError, null);
                    return;
                }
                const filteredSkillName = skillParser.filterUserSkillName(userSkillName);
                helper.gitCloneTemplate(cloneUrl, filteredSkillName, (cloneError) => {
                    callback(cloneError, cloneError ? null : { userSkillName, filteredSkillName });
                });
            });
        });
    });
}

function _postCommand(filteredSkillName, doDebug, profile, callback) {
    HookManager.createManager(CONSTANTS.HOOKS.CATEGORIES.POST_NEW, filteredSkillName, null, profile, doDebug, null, (err, hook) => {
        if (err) {
            callback(err, null);
            return;
        }
        hook.execute(callback);
    });
}

function changeSkillName(skillName, skillPath) {
    const manifest = jsonfile.readFileSync(skillPath);
    let locales = jsonUtility.getPropertyValueFromObject(manifest, ['manifest', 'publishingInformation', 'locales']);
    if (!locales) {
        locales = jsonUtility.getPropertyValueFromObject(manifest, ['skillManifest', 'publishingInformation', 'locales']);
    }
    Object.keys(locales).forEach((locale) => {
        locales[locale].name = skillName;
    });
    jsonfile.writeFileSync(skillPath, manifest, { spaces: 2 });
}

function adaptToInputProfileName(profile, configPath) {
    if (profile === 'default') {
        return;
    }
    const config = jsonUtility.read(configPath);
    let renamingConfig = jsonUtility.getPropertyValueFromObject(config, ['deploy_settings', 'default']);
    let renamingProfile = 'default';
    if (!renamingConfig) {
        if (Object.keys(config.deploy_settings).length === 0) {
            console.error('[Error]: Please provide a default profile in the skill template.');
            process.exit(1);
            return;
        }
        [renamingProfile] = Object.keys(config.deploy_settings);
        renamingConfig = jsonUtility.getPropertyValueFromObject(config, ['deploy_settings', renamingProfile]);
    }
    jsonUtility.insertObjectToObject(config, ['deploy_settings', profile], renamingConfig);
    jsonUtility.deletePropertyFromJsonObject(config, ['deploy_settings', renamingProfile]);
    jsonUtility.write(configPath, config);
}

function changeLambdaName(skillName, skillPath, lambdaName, profile, configPath) {
    // update uri's in merge fields
    const config = jsonfile.readFileSync(configPath);
    let mergeFieldApis = jsonUtility.getPropertyValueFromObject(config, ['deploy_settings', profile, 'merge', 'manifest', 'apis']);
    if (!mergeFieldApis) {
        mergeFieldApis = jsonUtility.getPropertyValueFromObject(config, ['deploy_settings', profile, 'merge', 'skillManifest', 'apis']);
    }
    if (mergeFieldApis) {
        for (const domain of Object.keys(mergeFieldApis)) {
            for (const key of Object.keys(mergeFieldApis[domain])) {
                if (!mergeFieldApis[domain][key]) {
                    continue;
                }
                if (key === 'endpoint') {
                    if (mergeFieldApis[domain][key].uri) {
                        mergeFieldApis[domain][key].uri = lambdaName || `ask-${domain}-${skillName}-${profile}`;
                    }
                } else if (key === 'regions') {
                    for (const region of Object.keys(mergeFieldApis[domain][key])) {
                        if (mergeFieldApis[domain][key][region].endpoint && mergeFieldApis[domain][key][region].endpoint.uri) {
                            mergeFieldApis[domain][key][region].endpoint.uri = lambdaName || `ask-${domain}-${skillName}-${profile}`;
                        }
                    }
                }
            }
        }
        jsonfile.writeFileSync(configPath, config, { spaces: 2 });
    }
    // update uri's in manifest
    const manifest = jsonfile.readFileSync(skillPath);
    let manifestApis = jsonUtility.getPropertyValueFromObject(manifest, ['manifest', 'apis']);
    if (!manifestApis) {
        manifestApis = jsonUtility.getPropertyValueFromObject(manifest, ['skillManifest', 'apis']);
    }
    if (manifestApis) {
        for (const domain of Object.keys(manifestApis)) {
            for (const key of Object.keys(manifestApis[domain])) {
                if (!manifestApis[domain][key]) {
                    continue;
                }
                if (key === 'endpoint') {
                    if (manifestApis[domain][key].uri) {
                        manifestApis[domain][key].uri = lambdaName || `ask-${domain}-${skillName}-${profile}`;
                    }
                } else if (key === 'regions') {
                    for (const region of Object.keys(manifestApis[domain][key])) {
                        if (manifestApis[domain][key][region].endpoint && manifestApis[domain][key][region].endpoint.uri) {
                            manifestApis[domain][key][region].endpoint.uri = lambdaName || `ask-${domain}-${skillName}-${profile}`;
                        }
                    }
                }
            }
        }
        jsonfile.writeFileSync(skillPath, manifest, { spaces: 2 });
    }
}
