const R = require('ramda');
const CONSTANTS = require('@src/utils/constants');

module.exports = {
    validateStage,
    validateLocale,
    validateUtterance,
    validateSkillId,
};

const localeRegex = /^[a-z]{2}-[A-Z]{2}$/;
const stagesWithoutCert = [CONSTANTS.SKILL.STAGE.DEVELOPMENT, CONSTANTS.SKILL.STAGE.LIVE];
const stages = [stagesWithoutCert, CONSTANTS.SKILL.STAGE.CERTIFICATION];

/**
 * General stage option validation logic
 * @param {String} stage
 * @param {Object} opts
 */
function validateStage(stage, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'stage';
    }

    validateWrapper(stage, opts, () => {
        validateString(stage);
        const allowCert = typeof opts.allowCert === 'undefined' ? false : opts.allowCert;
        const validStages = allowCert ? stages : stagesWithoutCert;
        const valid = validStages.indexOf(stage) !== -1;
        if (!valid) {
            throw (`Value must be in (${validStages.join(',')}).`);
        }
    });
}

/**
 * General locale option validation logic
 * @param {String} locale
 * @param {Object} opts
 */
function validateLocale(locale, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'locale';
    }

    validateWrapper(locale, opts, () => {
        validateString(locale);
        if (!localeRegex.test(locale)) {
            throw ('Invalid Format. Example: en-US.');
        }
    });
}

/**
 * General utterance option validation logic
 * @param {String} utterance
 * @param {Object} opts
 */
function validateUtterance(utterance, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'utterance';
    }

    validateWrapper(utterance, opts, () => {
        validateString(utterance);
    });
}

/**
 * General skill-id option validation logic
 * @param {String} skillId
 * @param {Object} opts
 */
function validateSkillId(skillId, opts = {}) {
    if (typeof opts.field === 'undefined') {
        opts.field = 'skill-id';
    }

    validateWrapper(skillId, opts, () => {
        validateString(skillId);
    });
}

function validateString(val) {
    if (!R.is(String, val)) {
        throw ('Must be a string.');
    }
    if (R.isEmpty(val.trim())) {
        throw ('Value must not be empty.');
    }
}

function validateWrapper(val, options = {}, callback) {
    const { field, required = true } = options;
    try {
        if (!R.isNil(val)) {
            if (callback) {
                callback();
            }
        } else if (required) {
            throw ('Field is required and must be set.');
        }
    } catch (err) {
        throw (`Please provide valid input for option: ${field}. ${err}`);
    }
}
