'use strict';

const path = require('path');
const fs = require('fs');
const apiWrapper = require('../api/api-wrapper.js');
const jsonUtility = require('../utils/json-utility');
const profileHelper = require('../utils/profile-helper');
const tools = require('../utils/tools');
const CONSTANTS = require('../utils/constants');
const displayUtil = require('../utils/display-utility');
const retry = require('retry');

// Public
module.exports = {
    createCommand: (commander) => {
        commander
            .command('simulate')
            .usage('<[-f|--file <file-path>] | [-t|--text <text>]> [-l|--locale <locale>] [-s|--skill-id <skill-id>] [-p|--profile <profile>] [--debug]')
            .description('simulate a user using your skill')
            .option('-f, --file <file-path>', 'path for simulate input file')
            .option('-t, --text <text>', 'text for utterance text')
            .option('-l, --locale <locale>', 'locale for the utterance text')
            .option('-s, --skill-id <skill-id>', 'skill-id for the skill')
            .option('-g, --stage <stage>', 'stage for the skill')
            .option('-p, --profile <profile>', 'ask cli profile')
            .option('--force-new-session', 'force simulate in a new skill session')
            .option('--debug', 'ask cli debug mode')
            .action(handle);

        function handle(options) {
            let skillId = options.skillId;

            if (!options.file && !options.text) {
                console.warn('Please input required parameter: file | text');
                return;
            }

            if (options.file && options.text) {
                console.warn('Both file and text parameters are specified. Please indicate which parameter you want.');
                return;
            }

            if (!options.locale && !process.env.ASK_DEFAULT_DEVICE_LOCALE) {
                console.warn('Please specify device locale via command line parameter locale or environment variable - ASK_DEFAULT_DEVICE_LOCALE');
                return;
            }

            if (options.file && !fs.existsSync(options.file)) {
                console.warn('Please verify the file exists');
                return;
            }

            let profile = profileHelper.runtimeProfile(options.profile);
            let locale = options.locale || process.env.ASK_DEFAULT_DEVICE_LOCALE;

            if (!skillId) {
                let projectConfigFile = path.join(process.cwd(), '.ask', 'config');
                if (!fs.existsSync(projectConfigFile)) {
                    console.warn('Failed to simulate. ' +
                        'Please run this command under the root of the skill project or explictly specify the skill id via skill-id option ');
                    return;
                } else {
                    let askConfig = jsonUtility.read(projectConfigFile);
                    skillId = jsonUtility.getPropertyValueFromObject(askConfig, ['deploy_settings', profile, 'skill_id']);
                    if (!skillId) {
                        console.warn('Failed to simulate. ' +
                            'The skill that you are trying to simulate has not been created or cloned. ' +
                            'If this is a new skill, please try creating and deploying the skill in your project first. ' +
                            'If this is an existing skill, please try cloning the skill in your project first. ' +
                            'Or you can try expliclty specifying the skill id via skill-id option.');
                        return;
                    }
                }
            }

            const stage = options.stage || CONSTANTS.SKILL.STAGE.DEVELOPMENT;
            apiWrapper.callSimulateSkill(CONSTANTS.COMMAND.NAME.SIMULATE, options.file, options.text, skillId, locale, options.forceNewSession, stage, profile, options.debug, (data) => {
                if (data.statusCode >= 300) {
                    console.error('Call simulate-skill error.');
                    console.error('Error code: ' + data.statusCode);
                    if (data.body && tools.convertDataToJsonObject(data.body)) {
                        console.error(JSON.stringify(tools.convertDataToJsonObject(data.body), null, 2));
                    }
                    return process.exit(1);
                }

                const response = tools.convertDataToJsonObject(data.body);
                if (response) {
                    const simulationId = response.id;
                    console.log('✓ Simulation created for simulation id: ' + simulationId);
                    pollingSimulationResult(simulationId, skillId, stage, profile, options.debug, (err, response) => {
                        if (err) {
                            console.error(err);
                            return process.exit(1);
                        }                             
                        return console.log(JSON.stringify(response, null, 2));
                    });
                }
            });
        }
    }
};

function pollingSimulationResult(simulationId, skillId, stage, profile, debug, callback) {
    const listenSpinner = displayUtil.Spinner('Waiting for the simulation response...');
    listenSpinner.start();

    keepPollingSimulationResult(simulationId, skillId, stage, profile, debug, (err, response) => {
        listenSpinner.stop();
        callback(err, response);
    });
}

function keepPollingSimulationResult(simulationId, skillId, stage, profile, debug, callback) {
    // max retry 20 times in 10 mins
    // which is a good estimate for skill status polling
    const operation = retry.operation({
        retries: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MAX_RETRY,
        minTimeout: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MIN_TIME_OUT,
        factor: CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.FACTOR
    });

    let isRetryFinished = false;
    operation.attempt(() => {
        if (operation.retry(isRetryFinished ? null : 'needRetry')) {
            apiWrapper.callGetSimulation(CONSTANTS.COMMAND.NAME.SIMULATE, simulationId, skillId, stage, profile, debug, (data) => {
                const response = tools.convertDataToJsonObject(data.body);
                // swallow SMAPI throttling exception. We don't want customer see throttling error, just keep trying polling.
                if (data.statusCode === 429) {
                    isRetryFinished = false;
                } else if (!response.hasOwnProperty('status')) {
                    callback(`[Error]: Unable to get skill simulation result for simulation id: ${simulationId}`, null);
                    isRetryFinished = true;
                } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.IN_PROGRESS) {
                    isRetryFinished = false;
                } else if (response.status === CONSTANTS.SKILL.SIMULATION_STATUS.SUCCESS ||
                    response.status === CONSTANTS.SKILL.SIMULATION_STATUS.FAILURE) {
                    callback(null, response);
                    isRetryFinished = true;
                } else {
                    callback('[Error]: Invalid response for skill simulation', null);
                    isRetryFinished = true;
                }
            });
        }

        if (operation.attempts() === CONSTANTS.CONFIGURATION.RETRY.GET_SIMULATE_STATUS.MAX_RETRY) {
            callback('[Error]: Polling timeout. The simulation request is still in processing.', null);
            isRetryFinished = true;
        }
    });
}
