const aws = require('aws-sdk');
const fs = require('fs');
const os = require('os');
const path = require('path');
const CONSTANT = require('@src/utils/constants');

module.exports = {
    initAWS,
    setRegionWithLambda,
    getCLICompatibleDefaultRegion
};

function initAWS(awsProfile) {
    if (!awsProfile) {
        console.error('[Error]: AWS credentials are not found in current profile.');
        process.exit(1);
    } else if (awsProfile === CONSTANT.PLACEHOLDER.ENVIRONMENT_VAR.AWS_CREDENTIALS) {
        let awsAccessKeyId;
        let awsSecretAccessKey;

        try {
            awsAccessKeyId = process.env.AWS_ACCESS_KEY_ID;
            awsSecretAccessKey = process.env.AWS_SECRET_ACCESS_KEY;
        } catch (error) {
            console.error(error.message);
            process.exit(1);
        }

        if (!awsAccessKeyId.trim()) {
            console.error('[Error]: Cannot find the environment variable: AWS_ACCESS_KEY_ID');
            process.exit(1);
        } else if (!awsSecretAccessKey.trim()) {
            console.error('[Error]: Cannot find the environment variable: AWS_SECRET_ACCESS_KEY');
            process.exit(1);
        }

        aws.config.credentials = new aws.Credentials(awsAccessKeyId, awsSecretAccessKey);
    } else {
        aws.config.credentials = new aws.SharedIniFileCredentials({
            profile: awsProfile
        });
    }
    aws.config.update({
        region: getCLICompatibleDefaultRegion(awsProfile) || CONSTANT.REGION_MAPPING.NA
    });
    return aws;
}

function setRegionWithLambda(awsInstance, arn) {
    const region = arn.match(/([a-z]{2})-([a-z]{4})([a-z]*)-\d{1}/g);
    awsInstance.config.update({
        region: region[0]
    });
    return awsInstance;
}

// We face the same issue as mentioned in https://github.com/aws/aws-sdk-js/issues/2181
// Issue is about when we instantiate the aws-sdk, the region value is not passed in correctly based on the profile.
// Temporary solution is to mimic aws-sdk's credential provider chain and wait for the solution.
class SharedIniFile {
    constructor(options) {
        options = options || {};
        this.isConfig = options.isConfig === true;
        this.filename = options.filename || this._getDefaultFilepath();
    }

    getProfile(profile) {
        const profileIndex = profile !== aws.util.defaultProfile && this.isConfig ? `profile ${profile}` : profile;
        return this._ensureFileLoaded()[profileIndex];
    }

    _getDefaultFilepath() {
        return path.join(os.homedir(), '.aws', this.isConfig ? 'config' : 'credentials');
    }

    _ensureFileLoaded() {
        const contents = fs.readFileSync(this.filename).toString();
        return aws.util.ini.parse(contents);
    }
}

function getCLICompatibleDefaultRegion(runtimeProfile) {
    const profile = runtimeProfile || process.env.AWS_PROFILE || process.env.AWS_DEFAULT_PROFILE || 'default';
    const toCheck = [
        {
            filename: process.env.AWS_SHARED_CREDENTIALS_FILE
        },
        {
            isConfig: true,
            filename: process.env.AWS_CONFIG_FILE
        }
    ];
    let region = process.env.AWS_REGION || process.env.AMAZON_REGION || process.env.AWS_DEFAULT_REGION || process.env.AMAZON_DEFAULT_REGION;
    while (!region && toCheck.length > 0) {
        const configFile = new SharedIniFile(toCheck.shift());
        const section = configFile.getProfile(profile);
        region = section && section.region;
    }
    return region;
}
