const fs = require('fs');
const async = require('async');
const retry = require('retry');

const domainRegistry = require('@src/utils/domain-registry');
const initAWS = require('@src/utils/init-aws');
const upload = require('@src/lambda/upload');
const displayUtil = require('@src/utils/display-utility');
const CONSTANTS = require('@src/utils/constants');

// Public
module.exports = {
    createLambda: (skillId, skillName, metaData, awsProfile, callback) => {
        const createLambdaSpinner = new displayUtil.Spinner(' Creating lambda function...');
        createLambdaSpinner.start();

        const roleNamePrefix = process.env.ASK_DEPLOY_ROLE_PREFIX || 'ask-lambda-';
        const roleName = roleNamePrefix + skillName.replace(/_/g, '-'); // IAM role doesn't allow underbar
        const aws = initAWS.initAWS(awsProfile);
        if (!aws) {
            createLambdaSpinner.stop();
            callback('[Error]: Cannot initialize aws');
            return;
        }
        // TODO: user provides IAM role
        const iamClient = new aws.IAM({ region: metaData.awsRegion });
        createIAMRole(iamClient, roleName, (err, arnData) => {
            if (err) {
                createLambdaSpinner.stop();
                callback(`[Error]: Create role error.\n${err}`);
                return;
            }
            attachRolePolicy(iamClient, roleName, (err) => {
                if (err) {
                    createLambdaSpinner.stop();
                    callback(`[Error]: Attach inline policy error.\n${err}`);
                    return;
                }
                createZip(metaData, (zipPath) => {
                    const lambdaClient = new aws.Lambda({ region: metaData.awsRegion });
                    createLambdaFunction(lambdaClient, metaData, zipPath, arnData.Role.Arn, (err, createResult) => {
                        if (err) {
                            createLambdaSpinner.stop();
                            callback(`[Error]: Create Lambda error.\n${err}`);
                            return;
                        }
                        addEventSource(skillId, lambdaClient, metaData, createResult, (err) => {
                            if (err) {
                                createLambdaSpinner.stop();
                                callback(`[Error]: Add event source error.\n${err}`);
                                return;
                            }
                            getLambdaFunctionRevisionId(lambdaClient, metaData, (err, newRevisionId) => {
                                if (err) {
                                    createLambdaSpinner.stop();
                                    callback(`[Error]: Get Lambda function revisionId error.\n${err}`);
                                    return;
                                }
                                createLambdaSpinner.stop();
                                createResult.revisionId = newRevisionId;
                                callback(null, createResult);
                            });
                        });
                    });
                });
            });
        });
    }
};

// Private
function createIAMRole(iamClient, roleName, callback) {
    const params = {
        RoleName: roleName
    };
    iamClient.getRole(params, (err, data) => {
        if (!err) {
            callback(null, data);
        } else {
            const policy = {
                Version: '2012-10-17',
                Statement: [
                    {
                        Effect: 'Allow',
                        Principal: {
                            Service: 'lambda.amazonaws.com'
                        },
                        Action: 'sts:AssumeRole'
                    }
                ]
            };
            params.AssumeRolePolicyDocument = JSON.stringify(policy);
            iamClient.createRole(params, (err, data) => {
                callback(err, data);
            });
        }
    });
}

function attachRolePolicy(iamClient, roleName, callback) {
    const params = {
        PolicyArn: 'arn:aws:iam::aws:policy/service-role/AWSLambdaBasicExecutionRole',
        RoleName: roleName
    };
    iamClient.attachRolePolicy(params, (err, data) => {
        callback(err, data);
    });
}

function createZip(metaData, callback) {
    upload.createZip(metaData.codeUri, (zipPath) => {
        callback(zipPath);
    });
}

function createLambdaFunction(lambdaClient, metaData, zipPath, roleArn, callback) {
    const params = {
        Code: {
            ZipFile: fs.readFileSync(zipPath)
        },
        FunctionName: metaData.functionName,
        Role: metaData.role || roleArn,
        Runtime: metaData.runtime,
        Handler: metaData.handler,
        MemorySize: metaData.memorySize || 128,
        Timeout: metaData.timeout || 8
    };
    const operation = retry.operation({
        retries: CONSTANTS.CONFIGURATION.RETRY.CREATE_LAMBDA.MAX_RETRY,
        minTimeout: CONSTANTS.CONFIGURATION.RETRY.CREATE_LAMBDA.MIN_TIME_OUT,
        factor: CONSTANTS.CONFIGURATION.RETRY.CREATE_LAMBDA.FACTOR
    });
    operation.attempt(() => {
        lambdaClient.createFunction(params, (err, functionData) => {
            if (operation.retry(err)) {
                return;
            }
            fs.unlinkSync(zipPath);
            if (err) {
                callback(err, null);
            } else {
                const result = {
                    arn: functionData.FunctionArn,
                    revisionId: functionData.RevisionId
                }
                callback(null, result);
            }
        });
    });
}

function addEventSource(skillId, lambdaClient, metaData, createResult, callback) {
    const domainSet = new Set();
    for (const usage of metaData.alexaUsage) {
        domainSet.add(usage.split('/')[0]);
    }
    const permissionsList = [];
    domainSet.forEach(domain => {
        const params = domainRegistry.getEventSourceParams(domain, skillId);
        params.FunctionName = createResult.arn;
        permissionsList.push(params);
    });

    async.forEach(permissionsList, (permissionParams, addCallback) => {
        lambdaClient.addPermission(permissionParams, (err) => {
            if (err) {
                addCallback(err);
            } else {
                addCallback();
            }
        });
    }, (error) => {
        callback(error);
    });
}

function getLambdaFunctionRevisionId(lambdaClient, metaData, callback) {
    const params = {
        FunctionName: metaData.functionName
    };
    lambdaClient.getFunction(params, (err, functionData) => {
        if (err) {
            callback(err);
            return;
        }
        callback(null, functionData.Configuration.RevisionId);
    });
}
