const fs = require('fs');

const displayUtil = require('../../utils/display-utility');
const upload = require('../../lambda/upload');
const initAWS = require('../init-aws');

module.exports = {
    updateLambda: (metaData, awsProfile, forceDeployment, callback) => {
        const updateSpinner = new displayUtil.Spinner(' Updating Lambda function...');
        updateSpinner.start();

        const aws = initAWS.initAWS(awsProfile);
        const revisionId = forceDeployment ? null : metaData.revisionId;
        createZip(metaData, (zipPath) => {
            const lambdaClient = new aws.Lambda({ region: metaData.awsRegion });
            updateLambdaFunction(lambdaClient, metaData, zipPath, revisionId, (err, updateResult) => {
                updateSpinner.stop();
                if (err) {
                    callback(`[Error]: Lambda update failed. Lambda ARN: ${metaData.arn}\n${err.message}`);
                } else {
                    callback(null, updateResult);
                }
            });
        });
    }
};

function createZip(metaData, callback) {
    upload.createZip(metaData.codeUri, (zipPath) => {
        callback(zipPath);
    });
}

function updateLambdaFunction(lambdaClient, metaData, zipPath, revisionId, callback) {
    _updateLambdaCode(lambdaClient, metaData, zipPath, revisionId, (updateCodeError, updateCodeResult) => {
        if (updateCodeError) {
            callback(updateCodeError, null);
            return;
        }
        if (updateCodeResult.Runtime !== metaData.runtime || updateCodeResult.Handler !== metaData.handler) {
            // if remote runtime/handler is not same as local runtime/handler then update configuration
            _updateLambdaConfiguration(lambdaClient, metaData, updateCodeResult.RevisionId, (updateConfigurationErr, updateConfigurationResult) => {
                if (updateConfigurationErr) {
                    callback(updateConfigurationErr, null);
                    return;
                }
                callback(null, { revisionId: updateConfigurationResult.RevisionId });
            });
        } else {
            // otherwise just callback the revisionId
            process.nextTick(() => {
                callback(null, { revisionId: updateCodeResult.RevisionId });
            });
        }
    });
}

function _updateLambdaCode(lambdaClient, metaData, zipPath, revisionId, callback) {
    const codeUpdateParams = {
        ZipFile: fs.readFileSync(zipPath),
        FunctionName: metaData.functionName
    };
    if (revisionId) {
        codeUpdateParams.RevisionId = revisionId;
    }
    lambdaClient.updateFunctionCode(codeUpdateParams, (updateCodeErr, functionData) => {
        fs.unlinkSync(zipPath);
        callback(updateCodeErr, functionData);
    });
}

function _updateLambdaConfiguration(lambdaClient, metaData, revisionId, callback) {
    const configurationUpdateParams = {
        FunctionName: metaData.functionName,
        Runtime: metaData.runtime,
        Handler: metaData.handler,
        RevisionId: revisionId
    };
    lambdaClient.updateFunctionConfiguration(configurationUpdateParams, (updateConfigurationErr, updateConfigurationResult) => {
        callback(updateConfigurationErr, updateConfigurationResult);
    });
}
