'use strict';

const expect = require('chai').expect;
const sinon = require('sinon');
const path = require('path');
const handler = require('@src/commands/new/handler');
const ui = require('@src/commands/new/ui');
const helper = require('@src/commands/new/helper');
const profileHelper = require('@src/utils/profile-helper');
const urlUtils = require('@src/utils/url-utils');
const skillParser = require('@src/utils/skill-parser');
const HookManager = require('@src/utils/hook-manager');
const CONSTANTS = require('@src/utils/constants');
const tools = require('@src/utils/tools');

describe('Commands new test - handler test', () => {
    const TEST_SKILL_NAME = 'skillName';
    const TEST_FILTER_SKILL_NAME = 'filteredSkillName';
    const TEST_GIT_URL = 'http://aaa.git';
    const TEST_INDEX_URL = 'http://test.json';
    const TEST_PROFILE = 'testProfile';
    const TEST_TEMPLATE_NAME = 'test';
    const TEST_DEBUG = true;
    const TEST_RUNTIME = 'Node.js 8.10';

    describe('# entry point checks', () => {
        beforeEach(() => {
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
        });

        it('| input operation is not a string, expect fatal error', () => {
            // setup
            const options = 'options';
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal('[Error]: Invalid command. Please run "ask new -h" for help.');
            expect(process.exit.args[0][0]).equal(1);
        });

        it('| when template name is mentioned without url, expect fatal error', () => {
            // setup
            const options = {
                template : TEST_TEMPLATE_NAME,
            };
            // call
            handler.handleOptions(options);
            //verify
            expect(console.error.args[0][0]).equal('[Error]: The template option only works when the templates link is specified in the url option.');
            expect(process.exit.args[0][0]).equal(1);
        });

        afterEach(() => {
            console.error.restore();
            process.exit.restore();
        });
    });

    describe('# clone with url and template', () => {
        let sandbox;
        beforeEach(() => {
            sandbox = sinon.createSandbox();
            sandbox.stub(profileHelper, 'runtimeProfile');
            sandbox.stub(urlUtils, 'isUrlWithGitExtension');
            sandbox.stub(skillParser, 'filterUserSkillName');
            sandbox.stub(handler, 'changeSkillName').callsFake(() => {});
            sandbox.stub(handler, 'adaptToInputProfileName').callsFake(() => {});
            sandbox.stub(handler, 'changeLambdaName').callsFake(() => {});
            sandbox.stub(path, 'join');
            sandbox.stub(tools,'v1ManifestTransformer');

            profileHelper.runtimeProfile.withArgs(TEST_PROFILE).returns(TEST_PROFILE);
            skillParser.filterUserSkillName.withArgs(TEST_SKILL_NAME).returns(TEST_FILTER_SKILL_NAME);
        });

        it('| when the url is git url, expect error message', () => {
            // setup
            const options = {
                url: TEST_GIT_URL,
                template: TEST_TEMPLATE_NAME
            };
            sinon.stub(console,'error');
            sinon.stub(process, 'exit');
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(true);
            //call
            handler.handleOptions(options);
            //verify
            expect(console.error.args[0][0]).equal('[Error]: Not allowed to specify the template name with the url pointing to a git repository. Please either use only url option with the link pointing to a git repository, or use templates link url along with the target template name.');
            expect(process.exit.args[0][0]).equal(1);
            // clear
            console.error.restore();
            process.exit.restore();
        });

        it('| when url is index url, make sure _cloneFromIndexUrlAndTemplate get called ', () => {
            // setup
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(false);
            const options = {
                url: TEST_INDEX_URL,
                template : TEST_TEMPLATE_NAME,
                debug: TEST_DEBUG
            };
            sinon.stub(helper,'getTemplateMap');
            // call
            handler.handleOptions(options);
            // verify
            expect(helper.getTemplateMap.args[0][0]).equal(TEST_INDEX_URL);
            expect(helper.getTemplateMap.args[0][1]).equal(TEST_DEBUG);
            // clear
            helper.getTemplateMap.restore();
        });

        it('| helper getTemplateMap returns error, expect console error', () => {
            // setup
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(false);
            const options = {
                url: TEST_INDEX_URL,
                template : TEST_TEMPLATE_NAME,
                debug: TEST_DEBUG
            };
            const GET_TEMPLATE_MAP_ERROR = 'getTemplateMapError';
            sinon.stub(helper,'getTemplateMap');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            helper.getTemplateMap.callsArgOnWith(2, {}, GET_TEMPLATE_MAP_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(GET_TEMPLATE_MAP_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| template name mentioned is not found in url, expect console error', () => {
            // setup
            urlUtils.isUrlWithGitExtension.withArgs(TEST_INDEX_URL).returns(false);
            sinon.stub(helper,'getTemplateMap');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');

            const options = {
                url: TEST_INDEX_URL,
                template : TEST_TEMPLATE_NAME
            };
            const TEST_TEMPLATE_MAP = {
                'temp' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(`[Error]: The template ${TEST_TEMPLATE_NAME} not found in given url. Please provide a valid template name.`);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| template name gitUrl field is undefined in url page, expect console error', () => {
            // setup
            urlUtils.isUrlWithGitExtension.withArgs(TEST_INDEX_URL).returns(false);
            sinon.stub(helper,'getTemplateMap');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');

            const options = {
                url: TEST_INDEX_URL,
                template : TEST_TEMPLATE_NAME
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {}
            };
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal('[Error]: Please make sure your template name has "gitUrl" field with the url pointing to the git repository.');
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| ui getSkillName returns error, expect console error', () => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                template : TEST_TEMPLATE_NAME
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };
            const GET_SKILL_NAME_ERROR = 'getSkillNameError';
            sinon.stub(helper, 'getTemplateMap');
            sinon.stub(ui, 'getSkillName');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            ui.getSkillName.callsArgOnWith(3, {}, GET_SKILL_NAME_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(GET_SKILL_NAME_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            ui.getSkillName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper gitCloneTemplate returns error, expect console error',() => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                template : TEST_TEMPLATE_NAME
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };
            const TEST_GIT_CLONE_TEMPLATE_ERROR = 'gitCloneTemplateError';
            sinon.stub(helper, 'getTemplateMap');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(ui, 'getSkillName');
            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, TEST_GIT_CLONE_TEMPLATE_ERROR);
            //call
            handler.handleOptions(options);
            //verify
            expect(console.error.args[0][0]).equal(TEST_GIT_CLONE_TEMPLATE_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            //clear
            helper.getTemplateMap.restore();
            helper.gitCloneTemplate.restore();
            ui.getSkillName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper gitCloneTemplate clones successfully, expect valid skillName', () => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                template: TEST_TEMPLATE_NAME,
                skillName: TEST_SKILL_NAME,
                profile: TEST_PROFILE
            };
            const TEST_TEMPLATE_MAP = {
                'test': {
                    'gitUrl' : TEST_GIT_URL
                }
            };
            sinon.stub(helper, 'getTemplateMap');
            sinon.stub(console, 'log');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(HookManager, 'createManager');
            sinon.stub(ui, 'getSkillName');
            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, null);
            //call
            handler.handleOptions(options);
            //verify
            expect(console.log.args[0][0]).equal(`Skill "${TEST_SKILL_NAME}" has been created based on the chosen template`);
            //clear
            helper.getTemplateMap.restore();
            helper.gitCloneTemplate.restore();
            ui.getSkillName.restore();
            HookManager.createManager.restore();
            console.log.restore();
        });
        afterEach(() => {
            sandbox.restore(); 
        });
    });

    describe('# clone with git url', () => {
        beforeEach(() => {
            sinon.stub(profileHelper, 'runtimeProfile');
            sinon.stub(urlUtils, 'isUrlWithGitExtension');
            sinon.stub(skillParser, 'filterUserSkillName');
            sinon.stub(tools, 'v1ManifestTransformer');
            sinon.stub(handler, 'changeSkillName');
            sinon.stub(handler, 'adaptToInputProfileName');
            sinon.stub(handler, 'changeLambdaName');

            profileHelper.runtimeProfile.withArgs(TEST_PROFILE).returns(TEST_PROFILE);
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(true);
            skillParser.filterUserSkillName.withArgs(TEST_SKILL_NAME).returns(TEST_FILTER_SKILL_NAME);
        });

        it('| make sure _cloneFromGitUrl is called in this case', () => {
            // setup
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_GIT_URL,
                profile: TEST_PROFILE
            };
            sinon.stub(ui, 'getSkillName');
            // call
            handler.handleOptions(options);
            // verify
            expect(ui.getSkillName.args[0][0]).equal(TEST_SKILL_NAME);
            expect(ui.getSkillName.args[0][1]).equal(TEST_GIT_URL);
            // clear
            ui.getSkillName.restore();
        });

        it('| ui getSkillName returns error, expect console error', () => {
            // setup
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_GIT_URL,
                profile: TEST_PROFILE
            };
            const GET_SKILL_NAME_ERROR = 'getSkillNameError';
            sinon.stub(ui, 'getSkillName');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            ui.getSkillName.callsArgOnWith(3, {}, GET_SKILL_NAME_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(GET_SKILL_NAME_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            ui.getSkillName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper gitCloneTemplate returns error, expect console error', () => {      
            // setup      
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_GIT_URL,
                profile: TEST_PROFILE
            };
            const TEST_GIT_CLONE_TEMPLATE_ERROR = 'gitCloneTemplateError';
            sinon.stub(ui, 'getSkillName');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            sinon.stub(helper, 'gitCloneTemplate');
            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, TEST_GIT_CLONE_TEMPLATE_ERROR);
            //call
            handler.handleOptions(options);
            //verify
            expect(console.error.args[0][0]).equal(TEST_GIT_CLONE_TEMPLATE_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            //clear
            ui.getSkillName.restore();
            helper.gitCloneTemplate.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper gitCloneTemplate clones successfully, expect valid skillName', () => {
            // setup
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_GIT_URL,
                profile: TEST_PROFILE
            };
            sinon.stub(ui, 'getSkillName');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(console, 'log');
            sinon.stub(HookManager, 'createManager');
            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, null);
            //call
            handler.handleOptions(options);
            //verify
            expect(console.log.args[0][0]).equal(`Skill "${TEST_SKILL_NAME}" has been created based on the chosen template`);
            //clear
            console.log.restore();
            ui.getSkillName.restore();
            helper.gitCloneTemplate.restore();
            HookManager.createManager.restore();
        });

        afterEach(() => {
            profileHelper.runtimeProfile.restore();
            urlUtils.isUrlWithGitExtension.restore();
            skillParser.filterUserSkillName.restore();
            handler.changeSkillName.restore();
            handler.adaptToInputProfileName.restore();
            handler.changeLambdaName.restore(); 
            tools.v1ManifestTransformer.restore();
        });
    });

    describe('# clone with index url', () => {
        beforeEach(() => {
            sinon.stub(profileHelper, 'runtimeProfile');
            sinon.stub(urlUtils, 'isUrlWithGitExtension');
            sinon.stub(skillParser, 'filterUserSkillName');
            sinon.stub(tools, 'v1ManifestTransformer');
            sinon.stub(handler, 'changeSkillName');
            sinon.stub(handler, 'adaptToInputProfileName');
            sinon.stub(handler, 'changeLambdaName');
            sinon.stub(urlUtils, 'isUrlWithJsonExtension');

            profileHelper.runtimeProfile.withArgs(TEST_PROFILE).returns(TEST_PROFILE);
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(false);
            urlUtils.isUrlWithJsonExtension.withArgs(TEST_INDEX_URL).returns(true);
            skillParser.filterUserSkillName.withArgs(TEST_SKILL_NAME).returns(TEST_FILTER_SKILL_NAME);

        });

        it('| make sure _cloneRepo is called', () => {
            // setup
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_INDEX_URL,
                debug: TEST_DEBUG
            };
            sinon.stub(helper,'getTemplateMap');
            // call
            handler.handleOptions(options);
            // verify
            expect(helper.getTemplateMap.args[0][0]).equal(TEST_INDEX_URL);
            expect(helper.getTemplateMap.args[0][1]).equal(TEST_DEBUG);
            //clear
            helper.getTemplateMap.restore();
        });

        it('| helper getTemplateMap returns error, expect console error', () => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                debug: TEST_DEBUG
            };
            const GET_TEMPLATE_MAP_ERROR = 'getTemplateMapError';
            sinon.stub(helper,'getTemplateMap');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            helper.getTemplateMap.callsArgOnWith(2, {}, GET_TEMPLATE_MAP_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(GET_TEMPLATE_MAP_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper getTemplateMap returns empty templateMap, expect console error', () => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                debug: TEST_DEBUG
            };            
            const TEST_TEMPLATE_MAP = '';
            sinon.stub(helper,'getTemplateMap');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal('[Error]: Cannot retrieve the template list at ' + TEST_INDEX_URL + '.');
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| ui getTargetTemplateName returns error, expect console error', () => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                debug: TEST_DEBUG
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };
            const TEST_TARGET_TEMPLATE_ERROR = 'targetTemplateError';

            sinon.stub(helper,'getTemplateMap');
            sinon.stub(ui,'getTargetTemplateName');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            ui.getTargetTemplateName.callsArgOnWith(2, {}, TEST_TARGET_TEMPLATE_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(TEST_TARGET_TEMPLATE_ERROR);
            expect(process.exit.args[0][0]).equal(1);          
            // clear
            helper.getTemplateMap.restore();
            ui.getTargetTemplateName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| gitUrl is not defined for the template, expect console error', () => {
            // setup
            const options = {
                url: TEST_INDEX_URL,
                debug: TEST_DEBUG
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : null
                }
            };

            sinon.stub(helper,'getTemplateMap');
            sinon.stub(ui,'getTargetTemplateName');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');
            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            ui.getTargetTemplateName.callsArgOnWith(2, {}, null, 'test');
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(`[Error]: gitUrl for test is not present in template map`);
            expect(process.exit.args[0][0]).equal(1);          
            // clear
            helper.getTemplateMap.restore();
            ui.getTargetTemplateName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| ui getSkillName returns error, expect console error', () => {
            // setup
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_INDEX_URL,
                profile: TEST_PROFILE
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };
            const GET_SKILL_NAME_ERROR = 'getSkillNameError';

            sinon.stub(helper,'getTemplateMap');
            sinon.stub(ui,'getTargetTemplateName');
            sinon.stub(ui, 'getSkillName');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');

            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            ui.getTargetTemplateName.callsArgOnWith(2, {}, null, 'test');
            ui.getSkillName.callsArgOnWith(3, {}, GET_SKILL_NAME_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(GET_SKILL_NAME_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            ui.getTargetTemplateName.restore();
            ui.getSkillName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper gitCloneTemplate returns error, expect console error', () => {
            // setup
            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_INDEX_URL,
                profile: TEST_PROFILE
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };

            sinon.stub(helper,'getTemplateMap');
            sinon.stub(ui,'getTargetTemplateName');
            sinon.stub(ui, 'getSkillName');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');

            const TEST_GIT_CLONE_TEMPLATE_ERROR = 'gitCloneTemplateError';

            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            ui.getTargetTemplateName.callsArgOnWith(2, {}, null, TEST_TEMPLATE_NAME);
            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, TEST_GIT_CLONE_TEMPLATE_ERROR);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(TEST_GIT_CLONE_TEMPLATE_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            helper.getTemplateMap.restore();
            helper.gitCloneTemplate.restore();
            ui.getTargetTemplateName.restore();
            ui.getSkillName.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| helper gitCloneTemplate clones succesfully, expect valid skillName', () => {
             // setup
             const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_INDEX_URL,
                profile: TEST_PROFILE
            };
            const TEST_TEMPLATE_MAP = {
                'test' : {
                    'gitUrl' : TEST_GIT_URL
                }
            };

            sinon.stub(helper,'getTemplateMap');
            sinon.stub(ui,'getTargetTemplateName');
            sinon.stub(ui, 'getSkillName');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(HookManager, 'createManager');
            sinon.stub(console, 'log');

            helper.getTemplateMap.callsArgOnWith(2, {}, null, TEST_TEMPLATE_MAP);
            ui.getTargetTemplateName.callsArgOnWith(2, {}, null, TEST_TEMPLATE_NAME);
            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.log.args[0][0]).equal(`Skill "${TEST_SKILL_NAME}" has been created based on the chosen template`);
            // clear
            helper.getTemplateMap.restore();
            helper.gitCloneTemplate.restore();
            HookManager.createManager.restore();
            ui.getTargetTemplateName.restore();
            ui.getSkillName.restore();
            console.log.restore();
        });

        afterEach(() => {
            profileHelper.runtimeProfile.restore();
            urlUtils.isUrlWithGitExtension.restore();
            urlUtils.isUrlWithJsonExtension.restore();
            skillParser.filterUserSkillName.restore();
            handler.changeSkillName.restore();
            handler.adaptToInputProfileName.restore();
            handler.changeLambdaName.restore(); 
            tools.v1ManifestTransformer.restore();
        });
    });

    describe('# clone template with user inputs', () => {
        beforeEach(() => {
            sinon.stub(tools, 'v1ManifestTransformer');
            sinon.stub(profileHelper, 'runtimeProfile');
            sinon.stub(urlUtils, 'isUrlWithGitExtension');
            sinon.stub(skillParser, 'filterUserSkillName');
            sinon.stub(handler, 'changeSkillName');
            sinon.stub(handler, 'adaptToInputProfileName');
            sinon.stub(handler, 'changeLambdaName');

            profileHelper.runtimeProfile.withArgs(TEST_PROFILE).returns(TEST_PROFILE);
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(false);
            skillParser.filterUserSkillName.withArgs(TEST_SKILL_NAME).returns(TEST_FILTER_SKILL_NAME);
        });

        it('| ui getRunTime returns error, expect console error', () => {
            // setup
            const options = {
                profile: TEST_PROFILE
            };
            const GET_RUNTIME_ERROR = 'getRunTimeError';
            
            sinon.stub(ui, 'getRunTime');
            sinon.stub(console, 'error');
            sinon.stub(process, 'exit');

            ui.getRunTime.callsArgOnWith(0, {}, GET_RUNTIME_ERROR, null);
            // call
            handler.handleOptions(options);
            // verify
            expect(console.error.args[0][0]).equal(GET_RUNTIME_ERROR);
            expect(process.exit.args[0][0]).equal(1);
            // clear
            ui.getRunTime.restore();
            console.error.restore();
            process.exit.restore();
        });

        it('| make sure _cloneRepo is called', () => {
            // setup
            sinon.stub(ui, 'getRunTime');
            sinon.stub(helper,'getTemplateMap');
            const options = {
                debug: TEST_DEBUG
            };
            ui.getRunTime.callsArgOnWith(0, {}, null, TEST_RUNTIME);
            // call
            handler.handleOptions(options);
            // verify
            expect(helper.getTemplateMap.args[0][0]).equal(CONSTANTS.LAMBDA.RUNTIME[TEST_RUNTIME]);
            expect(helper.getTemplateMap.args[0][1]).equal(TEST_DEBUG);
            // clear
            ui.getRunTime.restore();
            helper.getTemplateMap.restore();        
        });

        afterEach(() => {
            profileHelper.runtimeProfile.restore();
            urlUtils.isUrlWithGitExtension.restore();
            skillParser.filterUserSkillName.restore();
            handler.changeSkillName.restore();
            handler.adaptToInputProfileName.restore();
            handler.changeLambdaName.restore(); 
            tools.v1ManifestTransformer.restore();
        });
    });

    describe('# postCommand check', () => {
        beforeEach(() => {
            sinon.stub(profileHelper, 'runtimeProfile');
            sinon.stub(urlUtils, 'isUrlWithGitExtension');
            sinon.stub(skillParser, 'filterUserSkillName');
            sinon.stub(tools, 'v1ManifestTransformer');
            sinon.stub(handler, 'changeSkillName');
            sinon.stub(handler, 'adaptToInputProfileName');
            sinon.stub(handler, 'changeLambdaName');

            profileHelper.runtimeProfile.withArgs(TEST_PROFILE).returns(TEST_PROFILE);
            urlUtils.isUrlWithGitExtension.withArgs(TEST_GIT_URL).returns(true);
            skillParser.filterUserSkillName.withArgs(TEST_SKILL_NAME).returns(TEST_FILTER_SKILL_NAME);
        });

        it('| _postCommands createManager returns an error expect console error', () =>{
            //setup
            sinon.stub(ui, 'getSkillName');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(HookManager, 'createManager');
            sinon.stub(console, 'error');
            sinon.stub(console, 'log');
            sinon.stub(process, 'exit');

            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_GIT_URL,
                profile: TEST_PROFILE
            };
            const TEST_CREATE_HOOK_ERROR = 'cloneError';

            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, null);
            HookManager.createManager.callsArgOnWith(6, {}, TEST_CREATE_HOOK_ERROR, null);
            //call
            handler.handleOptions(options);
            //verify
            expect(console.error.args[0][0]).equal(TEST_CREATE_HOOK_ERROR);
            //clear
            ui.getSkillName.restore();
            helper.gitCloneTemplate.restore();
            HookManager.createManager.restore();
            console.error.restore();
            console.log.restore();
            process.exit.restore();
        });

        it('| _postCommands calls Hook execute method successfully', () =>{
            //setup
            sinon.stub(ui, 'getSkillName');
            sinon.stub(helper, 'gitCloneTemplate');
            sinon.stub(HookManager, 'createManager');
            sinon.stub(console,'log');

            const executeStub = sinon.stub();

            const options = {
                skillName: TEST_SKILL_NAME,
                url: TEST_GIT_URL,
                profile: TEST_PROFILE
            };

            ui.getSkillName.callsArgOnWith(3, {}, null, TEST_SKILL_NAME);
            helper.gitCloneTemplate.callsArgOnWith(2, {}, null);
            HookManager.createManager.callsArgOnWith(6, {}, null, {
                execute: executeStub
            });
            //call
            handler.handleOptions(options);
            //verify
            expect(executeStub.calledOnce).to.be.true;
            //clear
            ui.getSkillName.restore();
            console.log.restore();
            helper.gitCloneTemplate.restore();
            HookManager.createManager.restore();
        });
        
        afterEach(() => {
            profileHelper.runtimeProfile.restore();
            urlUtils.isUrlWithGitExtension.restore();
            skillParser.filterUserSkillName.restore();
            handler.changeSkillName.restore();
            handler.adaptToInputProfileName.restore();
            handler.changeLambdaName.restore(); 
            tools.v1ManifestTransformer.restore();
        });
    });
});
