'use strict';
const expect = require('chai').expect;
const sinon = require('sinon');
const tools = require('@src/utils/tools');
const helper = require('@src/commands/new/helper');
const ui = require('@src/commands/new/ui');
const httpClient = require('@src/clients/http-client');
const gitClient = require('@src/clients/git-client');
const displayUtility = require('@src/utils/display-utility');
const urlUtils = require('@src/utils/url-utils');

const noop = () => {};

describe('Commands new test - helper test', () => {
    const TEST_URL = 'https://test.com';
    const TEST_ERROR = 'TestError;'

    describe('# getTemplateMap check', () => {
        beforeEach(() => {
            sinon.stub(httpClient, 'request');
        });

        it('| httpClient request gets called successfully', () => {
            //setup
            const TEST_PARAMS = {
                url : TEST_URL,
                method : 'GET'
            };
            //call
            helper.getTemplateMap(TEST_URL, false, null);
            // verify
            expect(httpClient.request.args[0][0]).deep.equal(TEST_PARAMS);
        });
        
        it('| httpClient request returns error, expect error callback', (done) => {
            // setup
            httpClient.request.callsArgOnWith(3, {}, TEST_ERROR, null);
            // call
            helper.getTemplateMap(TEST_URL, false, (error, response) => {
                // verify
                expect(error).equal('[Error]: Cannot retrieve the template list.');
                expect(response).to.be.null;
                done();
            });
        });

        it('| httpClient request returns null status code, expect error callback ', (done) => {
            // setup
            const TEST_RESPONSE = {
                statusCode : null
            };
            httpClient.request.callsArgOnWith(3, {}, null, TEST_RESPONSE);
            // call
            helper.getTemplateMap(TEST_URL, false, (error, response) => {
                // verify
                expect(error).equal('[Error]: Cannot retrieve the template list.');
                expect(response).to.be.null;
                done();
            });
        });

        it('| httpClient request returns successfully, expect a response body', (done) => {
            // setup
            const TEST_RESPONSE = {
                statusCode: 200,
                body:'test'
            };
            sinon.stub(tools, "convertDataToJsonObject");
            tools.convertDataToJsonObject.returns(TEST_RESPONSE);
            httpClient.request.callsArgOnWith(3, {}, null, TEST_RESPONSE);
            // call
            helper.getTemplateMap(TEST_URL, false, (error, response) => {
                // verify
                expect(error).to.be.null;
                expect(response).equal(TEST_RESPONSE);
                done();
            })
        });

        afterEach(() => {
            httpClient.request.restore();
        });
    });

    describe('# gitCloneTemplate check', () => {
        const TEST_PROFILE = 'testProfile';
        const TEST_PACKAGE_NAME = 'testPackageName';
        const TEST_ERROR = 'testError';

        beforeEach(() => {
            const spinnerSpy = sinon.stub(displayUtility,"Spinner");    
            spinnerSpy.prototype.start = sinon.stub().returns(null);
            spinnerSpy.prototype.stop = sinon.stub().returns(null);
            sinon.stub(ui, 'getConfirmation');
            sinon.stub(console, 'warn');
            sinon.stub(urlUtils, 'isUrlOfficialTemplate').callsFake( () => true);
        });

        it('| check if git clone is called correctly', () => {
            // setup
            const gitStub = sinon.stub(gitClient, "clone");
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, TEST_PROFILE, noop );
            // verify
            expect(gitStub.args[0][0]).equal(TEST_URL);
            expect(gitStub.args[0][1].split('/').pop()).equal(TEST_PACKAGE_NAME);
            // clear   
            gitClient.clone.restore();
        });

        it('| git clone returns error, expect callback', () => {
            // setup
            sinon.stub(gitClient, "clone");
            gitClient.clone.callsArgOnWith(2, {}, TEST_ERROR);
            // call
            helper.gitCloneTemplate(TEST_URL, TEST_PACKAGE_NAME, (err) => {
                // verify
                expect(err).equal(TEST_ERROR);
            });
        });
        afterEach(() => {
            displayUtility.Spinner.restore(); 
            ui.getConfirmation.restore();  
            console.warn.restore(); 
            urlUtils.isUrlOfficialTemplate.restore();
        });
    });
});
