'use strict';

const expect = require('chai').expect;
const sinon = require('sinon');
const inquirer = require('inquirer');
const ui = require('@src/commands/new/ui');

const noop = () => {};

describe('Commands new - UI test', () => {
    const TEST_SKILL_NAME = 'testSkillName';
    const TEST_REPO_NAME = 'test';
    const TEST_URL = `https://${TEST_REPO_NAME}.git?data=1`;
    const TEST_RUN_TIME = 'testRunTime';
    
    describe('# getSkillName check', () => {
        beforeEach(() => {
            sinon.stub(inquirer,'prompt');
        });

        it('| getSkillName is defined by user return correctly', (done) => {
            // call
            ui.getSkillName(TEST_SKILL_NAME, TEST_URL, null, (err, response) => {
                // verify
                expect(err).to.be.null;
                expect(response).equal(TEST_SKILL_NAME);
                done();
            });
        });

        it('| repoName is retrieved correctly', () => {
            // setup
            inquirer.prompt.resolves({ skillName : TEST_SKILL_NAME});
            // call
            ui.getSkillName(null, TEST_URL, null, noop);
            // verify
            expect(inquirer.prompt.args[0][0][0]['default']).equal(TEST_REPO_NAME);
        });

        it('| getSkillName retrieves user input correctly', (done) => {
            // setup
            inquirer.prompt.resolves({skillName : TEST_SKILL_NAME});
            // call
            ui.getSkillName(null, null, null, (err, response) => {
                // verify
                expect(response).equal(TEST_SKILL_NAME);
                done();
            });
        });

        afterEach(() => {
            inquirer.prompt.restore();
        });
    });

    describe('# getRunTime check', () => {
        beforeEach(() => {
            sinon.stub(inquirer,'prompt');
        });

        it('| runtTime entered by user is retrived correctly', (done) => {
            // setup
            inquirer.prompt.resolves({runTime : TEST_RUN_TIME});
            // call
            ui.getRunTime((err, response) => {
                // verify
                expect(err).to.be.null;
                expect(response).equal(TEST_RUN_TIME);
                done();
            });
        });

        afterEach(() => {
            inquirer.prompt.restore();
        });
    });

    describe('# getTargetTemplate check', () => {
        const TEST_TEMPLATE_LIST = ['one','two','three'];
       
        beforeEach(() => {
            sinon.stub(inquirer,'prompt');            
        });

        it('| templateName not found in list of templates, returns error', (done) => {
            // setup
            const TEST_TEMPLATE_NAME = 'test';
            // call
            ui.getTargetTemplateName(TEST_TEMPLATE_LIST, TEST_TEMPLATE_NAME, (err, response) => {
                // verify
                expect(err).equal('[Error]: No template found with the input template name. Please make sure the template list contains the one you want to use.');
                done();
            });
        });

        it('| templateName is found in list of templates, returns correct value', (done) => {
            // setup
            const TEST_TEMPLATE_NAME = 'two';
            // call
            ui.getTargetTemplateName(TEST_TEMPLATE_LIST, TEST_TEMPLATE_NAME, (err, response) => {
                // verify
                expect(err).to.be.null;
                expect(response).equal(TEST_TEMPLATE_NAME);
                done();
            });
        });

        it('| template name entered by user is retrived correctly', (done) => {
            // setup
            const TEST_TEMPLATE_NAME = 'two';
            inquirer.prompt.resolves({templateName : TEST_TEMPLATE_NAME});
            // call
            ui.getTargetTemplateName(TEST_TEMPLATE_LIST, null, (err, response) => {
                // verify
                expect(err).to.be.null;
                expect(response).equal(TEST_TEMPLATE_NAME);
                done();
            });
        });

        afterEach(() => {
            inquirer.prompt.restore();
        });
    });
});