'use strict';

const expect = require('chai').expect;
const CONSTANTS = require('@src/utils/constants');
const optionValidator = require('@src/commands/option-validator');


describe('option validation testing', () => {
    describe('# validate stage', () => {
        it('| should match development stage', () => {
            expect(() => optionValidator.validateStage(CONSTANTS.SKILL.STAGE.DEVELOPMENT)).to.not.throw();
        });

        it('| should match live stage', () => {
            expect(() => optionValidator.validateStage(CONSTANTS.SKILL.STAGE.LIVE)).to.not.throw();
        });

        it('| should expect strict casing', () => {
            expect(() => optionValidator.validateStage(CONSTANTS.SKILL.STAGE.DEVELOPMENT.toUpperCase()))
                .to.throw('Please provide valid input for option: stage. Value must be in (development,live).');
        });

        it('| should match cert when allowed', () => {
            expect(() => optionValidator.validateStage(CONSTANTS.SKILL.STAGE.CERTIFICATION, {allowCert: true}))
                .to.not.throw();
        });

        it('| should not match cert when not allowed', () => {
            expect(() => optionValidator.validateStage(CONSTANTS.SKILL.STAGE.CERTIFICATION, {allowCert: false}))
                .to.throw('Please provide valid input for option: stage. Value must be in (development,live).');
        });

        validatorTests(optionValidator.validateStage, 'stage');
    });

    describe('# validate locale', () => {
        it('| should allow valid format', () => {
            expect(() => optionValidator.validateLocale('en-US')).to.not.throw();
        });

        it('| should be case sensitive', () => {
            expect(() => optionValidator.validateLocale('En-uS'))
                .to.throw('Please provide valid input for option: locale. Invalid Format. Example: en-US.');
        });

        it('| should not allow language only', () => {
            expect(() => optionValidator.validateLocale('en'))
                .to.throw('Please provide valid input for option: locale. Invalid Format. Example: en-US.');
        });

        it('| should not allow invalid format', () => {
            expect(() => optionValidator.validateLocale('en_US'))
                .to.throw('Please provide valid input for option: locale. Invalid Format. Example: en-US.');
        });

        it('| should not allow whitespace', () => {
            expect(() => optionValidator.validateLocale(' en-us'))
                .to.throw('Please provide valid input for option: locale. Invalid Format. Example: en-US.');
            expect(() => optionValidator.validateLocale('en-us '))
                .to.throw('Please provide valid input for option: locale. Invalid Format. Example: en-US.');
        });

        validatorTests(optionValidator.validateLocale, 'locale');
    });

    describe('# validate utterance', () => {
        it('| should allow valid format', () => {
            expect(() => optionValidator.validateUtterance('I am an utterance')).to.not.throw();
        });

        validatorTests(optionValidator.validateUtterance, 'utterance');
    });

    describe('# validate skillId', () => {
        it('| should allow valid format', () => {
            expect(() => optionValidator.validateSkillId('amz1.ask.something')).to.not.throw();
        });

        validatorTests(optionValidator.validateSkillId, 'skill-id');
    });

    function validatorTests(func, field) {
        it('| should not succeed on null or empty', () => {
            const valueEmptyErr = `Please provide valid input for option: ${field}. Value must not be empty.`;
            const requiredErr = `Please provide valid input for option: ${field}. Field is required and must be set.`;
            expect(() => func(null)).to.throw(requiredErr);
            expect(() => func()).to.throw(requiredErr);
            expect(() => func('')).to.throw(valueEmptyErr);
            expect(() => func('           ')).to.throw(valueEmptyErr);
        });

        it('| should override field name', () => {
            expect(() => func(null, { field: 'test' }))
                .to.throw('Please provide valid input for option: test. Field is required and must be set.');
        });

        it('| should succeed when not required', () => {
            expect(() => func(null, { required: false })).to.not.throw();
            expect(() => func(undefined, { required: false })).to.not.throw();
            expect(() => func('', { required: false })).to.throw();
            expect(() => func('   ', { required: false })).to.throw();
        });
    }
});
