'use strict';

const chai = require('chai');
const expect = chai.expect;
const sinon = require('sinon');
const fs =require('fs');
const jsonUtility = require('../../../lib/utils/json-utility');
const profileHelper = require('../../../lib/utils/profile-helper');
const validator = require('../../../lib/dialog/dialog-validator');

describe('dialog command testing', () => {
    describe('# dialog input options testing', () => {
        let sandbox;
        let command;
        let exitStub;
        let testInputOption;

        before(() => {
        });

        beforeEach(() => {
            sandbox = sinon.sandbox.create();
            sandbox.stub(console, 'error');
            exitStub = sandbox.stub(process, 'exit');
        });

        afterEach(() => {
            sandbox.restore();
        });

        it ('| error if locale is not set and ASK_DEFAULT_DEVICE_LOCALE is not set and no replay file', () => {
            sandbox.stub(process.env.ASK_DEFAULT_DEVICE_LOCALE).returns(null);
            testInputOption = {
                skillId: "skillId",
                locale: null,
                output: null,
                replay: null,
                profile: null,
                debug: false
            };
            validator.validateInputOption(testInputOption);            
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Please specify device locale via any of the approaches below: \r\n - command line parameter "--locale"'
                + ' \r\n - environment variable "ASK_DEFAULT_DEVICE_LOCALE"'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);
        });

        it ('| error if project config file does not exist', () => {
            sandbox.stub(fs, 'existsSync').returns(false);
            sandbox.stub(profileHelper, 'runtimeProfile').returns('default');

            testInputOption = {
                skillId: null,
                locale: "locale",
                output: null,
                replay: null,
                profile: null,
                debug: false
            };
            validator.validateInputOption(testInputOption);  
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Failed to simulate dialog. Run this command in the root directory of the skill project, or explictly specify the skill-id via the skill-id option.'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);
        });

        it ('| error if project file exists but skill id doesn not exist in the config file and no replay file', () => {
            sandbox.stub(fs, 'existsSync').returns(true);
            sandbox.stub(jsonUtility, 'read').returns('askConfig');
            sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns(null);
            sandbox.stub(profileHelper, 'runtimeProfile').returns('default');

            testInputOption = {
                skillId: null,
                locale: "locale",
                output: null,
                replay: null,
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption);  
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Failed to simulate dialog. ' +
                'The skill that you are trying to simulate has not been created or cloned. ' +
                'If this is a new skill, please try creating and deploying the skill in your project first. ' +
                'If this is an existing skill, please try cloning the skill in your project first. ' +
                'You can also specify the skill id explicitly via the skill-id option.'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);

        });

        it ('| error if the input replay file does not exsit', () => {
            sandbox.stub(fs, 'existsSync').returns(false);
            sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns(null);
            
            testInputOption = {
                skillId: null,
                locale: null,
                output: null,
                replay: "replayFile",
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption); 
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Please verify the file exists.'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);
        }); 

        it ('| error if has replay file but no locale in the file, no locale is set via command line, and no ASK_DEFAULT_DEVICE_LOCALE is not set', () => {
            sandbox.stub(fs, 'existsSync').returns(true);
            sandbox.stub(jsonUtility,'read').returns('replayFile');
            sandbox.stub(profileHelper, 'runtimeProfile').returns('default');
            sandbox.stub(jsonUtility, 'getProperty')
                   .onCall(0)
                   .returns('text')
                   .onCall(1)
                   .returns('skillId')
                   .onCall(2)
                   .returns(null);
            sandbox.stub(process.env.ASK_DEFAULT_DEVICE_LOCALE).returns(null);

            testInputOption = {
                skillId: null,
                locale: null,
                output: null,
                replay: "replayFile",
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption); 
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Please specify device locale via any of the approaches below:'
                + ' \r\n - replay file field "locale"'
                + ' \r\n - environment variable "ASK_DEFAULT_DEVICE_LOCALE"'
            );
        });

        it ('| error if has replay file but no skillId in the file, no skill id is set via command line, no project file',() => {
            sandbox.stub(fs, 'existsSync')
                   .onFirstCall()
                   .returns(true)
                   .onSecondCall()
                   .returns(false);
            sandbox.stub(profileHelper, 'runtimeProfile').returns('default');
            sandbox.stub(process.env.ASK_DEFAULT_DEVICE_LOCALE).returns(null);

            sandbox.stub(jsonUtility,'read').returns('replayFile');
            sandbox.stub(jsonUtility, 'getProperty')
                   .onCall(0)
                   .returns("text")
                   .onCall(1)
                   .returns(null)
                   .onCall(2)
                   .returns("en-US");

            testInputOption = {
                skillId: null,
                locale: null,
                output: null,
                replay: "replayFile",
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption); 
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Failed to simulate dialog. Run this command in the root directory of the skill project, or explictly specify the skill-id in the replay file.'
            );
        });

        it ('| error if output file name is invalid', () => {
            testInputOption = {
                skillId: "skillId",
                locale: "en-US",
                output: ".?testOutput",
                replay: "replayFile",
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption); 
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Input a valid output file name or file path.'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);
        });

        it ('| error if set skill both through command line and replay file', () => {
            sandbox.stub(fs, 'existsSync').returns(true);
            sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns('inputs');
            testInputOption = {
                skillId: "skillId",
                locale: null,
                output: null,
                replay: "replayFile",
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption); 
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: With the replay option, the replay file contains the skill-id used for the simulation. Do not otherwise specify the skill-id.'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);
        });

        it ('| error if set locale both through command line and replay file', () => {
            sandbox.stub(fs, 'existsSync').returns(true);
            sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns('inputs');
                  
            testInputOption = {
                skillId: null,
                locale: "en-US",
                output: null,
                replay: "replayFile",
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption); 
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: With the replay option, the replay file contains the locale used for the simulation. Do not otherwise specify the locale.'
            );
            expect(exitStub.getCall(0).args[0]).equal(1);
        });

        it ('| error if project file exists but skill id doesn not exist in the config file', () => {
            sandbox.stub(fs, 'existsSync').returns(true);
            sandbox.stub(jsonUtility, 'read').returns('askConfig');
            sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns(null);
            sandbox.stub(profileHelper, 'runtimeProfile').returns('default');

            testInputOption = {
                skillId: null,
                locale: null,
                output: null,
                replay: null,
                profile: "default",
                debug: false
            };
            validator.validateInputOption(testInputOption);
            expect(console.error.getCall(0).args[0]).equal(
                '[Error]: Failed to simulate dialog. ' +
                'The skill that you are trying to simulate has not been created or cloned. ' +
                'If this is a new skill, please try creating and deploying the skill in your project first. ' +
                'If this is an existing skill, please try cloning the skill in your project first. ' +
                'You can also specify the skill id explicitly via the skill-id option.'
            );
        });
    });
});
