'use strict';

const expect = require('chai').expect;
const sinon = require('sinon');
const fs =require('fs');

const MockCommand = require('../mocks/command');
const validateCommand = require('../../../lib//validate/validate');
const jsonUtility = require('../../../lib/utils/json-utility');
const profileHelper = require('../../../lib/utils/profile-helper');
const apiWrapper = require('../../../lib/api/api-wrapper.js');
const tools = require('../../../lib/utils/tools');

describe('validate command testing', () => {
    describe('# build high-level command validate', () => {
        let sandbox;
        let command;

        before(() => {
        });

        beforeEach(() => {
            command = new MockCommand();
            validateCommand.createCommand(command);
            sandbox = sinon.sandbox.create();
            sandbox.stub(console, 'warn');
        });

        afterEach(() => {
            sandbox.restore();
        });

        it('| warn if locale is not set and ASK_DEFAULT_DEVICE_LOCALE is not set', () => {
            sandbox.stub(process.env.ASK_DEFAULT_DEVICE_LOCALE).returns(null);
            command.runWith('validate -g development');
            expect(console.warn.getCall(0).args[0]).equal(
                'Please specify locale to validate for your skill via command line parameter <-l|--locales> or environment variable - ASK_DEFAULT_DEVICE_LOCALE'
            );
        });

        it ('| warn if project config file does not exist', () => {
            sandbox.stub(fs, 'existsSync').returns(false);
            sandbox.stub(profileHelper, 'runtimeProfile').returns('profile');

            command.runWith('validate -g development -l en-us');
            expect(console.warn.getCall(0).args[0]).equal(
                'Failed to validate. ' +
                'Please run this command under the root of the skill project or explictly specify the skill id via skill-id option '
            );
        });

        it ('| warn if project file exists but skill id doesn not exist in the config file', () => {
            sandbox.stub(fs, 'existsSync').returns(true);
            sandbox.stub(jsonUtility, 'read').returns('askConfig');
            sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns(null);
            sandbox.stub(profileHelper, 'runtimeProfile').returns('profile');

            command.runWith('validate -g development -l en-us');
            expect(console.warn.getCall(0).args[0]).equal(
                'Failed to validate. ' +
                'The skill that you are trying to validate has not been created or cloned. ' +
                'If this is a new skill, please try creating and deploying the skill in your project first. ' +
                'If this is an existing skill, please try cloning the skill in your project first. ' +
                'Or you can try expliclty specifying the skill id via skill-id option.'
            );
        });

        describe('make all the way through to make a apiWrapper call', () => {
            beforeEach(() => {
                sandbox.stub(fs, 'existsSync').returns(true);
                sandbox.stub(jsonUtility, 'read').returns('askConfig');
                sandbox.stub(jsonUtility, 'getPropertyValueFromObject').returns('12345');
                sandbox.stub(profileHelper, 'runtimeProfile').returns('profile');
                sandbox.stub(apiWrapper, 'callValidateSkill')
                    .callsArgWith(5, {"body": {"id": 6789, "status": "IN_PROGRESS"}})
            });

            it ('| polling successful', () => {
                sandbox.stub(apiWrapper, 'callGetValidation')
                    .callsArgWith(5, {"body": {"id": 6789, "status": "SUCCESSFUL"}});
                let clock = sandbox.useFakeTimers();

                sandbox.stub(console, 'log');
                sandbox.stub(console, 'error');

                command.runWith('validate -g live -l en-us');
                clock.tick(1001);

                expect(apiWrapper.callValidateSkill.getCall(0).args[0]).equal('12345');
                expect(apiWrapper.callValidateSkill.getCall(0).args[1]).equal('en-us');
                expect(apiWrapper.callValidateSkill.getCall(0).args[2]).equal('live');
                expect(apiWrapper.callValidateSkill.getCall(0).args[3]).equal('profile');

                expect(console.error.getCall(0).args[0]).equal(
                    '✓ Validation created for validation id: 6789'
                );

                let obj = tools.convertDataToJsonObject('{"id": 6789, "status": "SUCCESSFUL"}');
                let str = JSON.stringify(obj, null, 2);
                expect(console.log.getCall(0).args[0]).equal(str);

                // Need to restore stubbed console.log such that mocha can display test run status to the console.
                console.log.restore();
            });

            it ('| polling failed because the response does not have status field', () => {
                sandbox.stub(apiWrapper, 'callGetValidation')
                    .callsArgWith(5, {"body": {}});
                let clock = sandbox.useFakeTimers();

                sandbox.stub(console, 'error');

                command.runWith('validate -l en-us');
                clock.tick(1001);
                clock.tick(1001);

                expect(apiWrapper.callValidateSkill.getCall(0).args[0]).equal('12345');
                expect(apiWrapper.callValidateSkill.getCall(0).args[1]).equal('en-us');
                expect(apiWrapper.callValidateSkill.getCall(0).args[2]).equal('development');
                expect(apiWrapper.callValidateSkill.getCall(0).args[3]).equal('profile');

                expect(console.error.getCall(0).args[0]).equal(
                    '✓ Validation created for validation id: 6789'
                );

                expect(console.error.getCall(1).args[0]).equal(
                    '𐄂 Unable to get skill validation result for validation id: 6789'
                );
            });

            it ('| polling failed because the response has invalid status', () => {
                sandbox.stub(apiWrapper, 'callGetValidation')
                    .callsArgWith(5, {"body": {"id": 6789, "status": "INVALID"}});
                let clock = sandbox.useFakeTimers();

                sandbox.stub(console, 'error');

                command.runWith('validate -l en-us');
                clock.tick(1001);
                clock.tick(1001);

                expect(apiWrapper.callValidateSkill.getCall(0).args[0]).equal('12345');
                expect(apiWrapper.callValidateSkill.getCall(0).args[1]).equal('en-us');
                expect(apiWrapper.callValidateSkill.getCall(0).args[2]).equal('development');
                expect(apiWrapper.callValidateSkill.getCall(0).args[3]).equal('profile');

                expect(console.error.getCall(0).args[0]).equal(
                    '✓ Validation created for validation id: 6789'
                );

                expect(console.error.getCall(1).args[0]).equal(
                    'x [Error]: Invalid response for skill validation'
                );
            });
        });
    });
});
